#!/usr/bin/env python
# -*- coding: UTF-8 -*-

#Licence: GPLv3.0
#Copyright: Krasimir S. Stefanov

import os
import platform
import locale
import sys
import os.path
import ConfigParser
import gettext
import pygtk
pygtk.require("2.0")
import gtk
import gtk.glade
import gobject
import re
import pango
import asubprocess
import signal
import random
import cairo
import atk
import pangocairo
import shlex
import webbrowser
import gtk.gdk
from optparse import OptionParser

# localization
APP = "vidrop"
APP_VERSION = "0.6.8"
if platform.system() == "Windows":
	DIR = "locale"
	os.environ['LANG'] = locale.getdefaultlocale()[0]
	os.environ['LANGUAGE'] = locale.getdefaultlocale()[0]
else:
	if os.path.exists('./locale'):
		DIR = os.path.join(os.path.dirname(sys.argv[0]),"locale")
	else:
		DIR = "/usr/share/locale"
locale.setlocale(locale.LC_ALL, '')
gtk.glade.bindtextdomain(APP, DIR)
gtk.glade.textdomain(APP)
gettext.bindtextdomain(APP, DIR)
gettext.textdomain(APP)
gettext.install(APP, localedir=DIR)
global _
_ = gettext.gettext
# /localization

supported_container_map = {
    'Ogg':        [ 'vorbis', 'theora', 'flac', 'speex', 'celt', 'dirac' ],
    'MXF':        [ 'mp3', 'h264', 'aac', 'ac3', 'mpeg2', 'mpeg4' ],
    'Matroska':   [ 'flac', 'dirac', 'aac', 'ac3', 'theora', 'mp3', 'h264', 'mpeg4', 'mpeg2', 'xvid', 'vorbis', 'h263p' ],
    'AVI':        [ 'mp3', 'h264', 'dirac', 'ac3', 'mpeg2', 'mpeg4', 'xvid','wma2','wmv2' ],
    'Quicktime':  [ 'aac', 'h264', 'ac3', 'dirac', 'mp3', 'mpeg2', 'mpeg4' ],
    'MPEG4':      [ 'aac', 'h264', 'mp3', 'mpeg2', 'mpeg4' ],
    '3GPP':       [ 'aac', 'h264', 'mp3', 'mpeg2', 'mpeg4','amrnb','h263p' ],
    'MPEG PS':    [ 'mp3', 'mpeg2', 'ac3', 'h264', 'aac', 'mpeg4' ],
    'MPEG TS':    [ 'mp3', 'h264', 'ac3', 'mpeg2', 'aac', 'mpeg4', 'dirac' ],
    'FLV':        [ 'mp3', 'h264' ],
    'ASF':        [ 'wma2','wmv2','mp3']
}


video_codecs = {

	'copy'		: {
		'fourcc':	[]
		, 'lib'	:	'copy'
		, 'settings' : ''
		, 'ext' : ''
	}
	#, 'novideo'		: {
	#	'fourcc':	[]
	#	, 'lib'	:	'novideo'
	#}
	, 'not-set'		: {
		'fourcc':	[]
		, 'lib'	:	'not-set'
		, 'settings' : ''
		, 'ext' : ''
	}
	#, 'raw'		: ['raw ']
	#, 'nuv'		: ['nuv']
	#, 'qtvideo'	: []
	#, 'libdv'	: ['CDVC', 'CUV5', 'CDVH']
	, 'xvid'	: {
		'fourcc':	['XVID', 'FFDS', 'FVFW', 'DX50', 'DIVX', 'MP4V']
		, 'lib'	:	'xvid'
		, 'options' :	{
			'me_quality'	:	['int-range', 5 ,1 ,6] # type,default,min,max
			, 'max_bframes'	:	['int', 0]
			, 'quant_type'	:	['list','h263',['h263']]
			, 'turbo'		:	['toggle',True] # hide on false
			, 'nochroma_me'	:	['toggle',True]
			, 'notrellis'	:	['toggle',True]
			, 'vhq'			:	['boolean',1] # show allways, 0 or 1
		}
		, 'settings' : ':max_bframes=0:quant_type=h263:turbo:nochroma_me:notrellis:vhq=0'
		, 'ext' : 'xvid'
	}
	, 'x264'	: {
		'fourcc':	['H264', 'X264', 'AVC']
		, 'lib'	:	'x264'
		, 'settings' : ':bframes=0'
		, 'ext' : 'h264'
	}
	#lavc
	, 'mjpeg'	: {
		'fourcc':	['MJPG', 'AVDJ', 'AVID', 'AVRn', 'dmb1', 'IJPG', 'mjpa', 'mjpb' ]
		, 'lib'	:	'lavc'
		, 'settings' : ':vmax_b_frames=0'
		, 'ext' : 'mjpeg'
	}
	, 'ljpeg'	: {
		'fourcc':	['LJPG']
		, 'lib'	:	'lavc'
		, 'settings' : ':vmax_b_frames=0'
		, 'ext' : 'ljpeg'
	}
	, 'jpegls'	: {
		'fourcc':	[]
		, 'lib'	:	'lavc'
		, 'settings' : ':vmax_b_frames=0'
		, 'ext' : 'jpegls'
	}
	#, 'targa'	: {
	#	'fourcc':	[]
	#	, 'lib'	:	'lavc'
	#}
	#, 'gif'		: {
	#	'fourcc':	[]
	#	, 'lib'	:	'lavc'
	#}
	#, 'bmp'		: {
	#	'fourcc':	[]
	#	, 'lib'	:	'lavc'
	#}
	#, 'png'		: {
	#	'fourcc':	[]
	#	, 'lib'	:	'lavc'
	#}
	, 'h261'	: {
		'fourcc':	['H261', 'D261', 'L261', 'M261', 'V261' ]
		, 'lib'	:	'lavc'
		, 'settings' : ':vmax_b_frames=0'
		, 'ext' : 'h261'
	}
	, 'h263'	: {
		'fourcc':	['H263', 'D263', 'L263', 'M263', 'S263', 'T263', 'U263', 'X263']
		, 'lib'	:	'lavc'
		, 'settings' : ':vmax_b_frames=0'
		, 'ext' : 'h263'
	}
	, 'h263p'	: {
		'fourcc':	['H263', 'D263', 'L263', 'M263', 'S263', 'T263', 'U263', 'X263']
		, 'lib'	:	'lavc'
		, 'settings' : ':vmax_b_frames=0'
		, 'ext' : 'h263p'
	}
	, 'mpeg4'	: {
		'fourcc':	['XVID', 'FFDS', 'FVFW', 'DX50', 'DIVX', 'MP4V', '3IV2', '3IVX', 'MP4S', 'M4S2', 'RMP4', 'DM4V', 'DXGM', 'WV1F', 'FMP4', 'HDX4']
		, 'lib'	:	'lavc'
		, 'settings' : ':vmax_b_frames=0'
		, 'ext' : 'mpeg4'
	}
	, 'msmpeg4'	: {
		'fourcc':	['DIV3', 'MP43', 'COL1', 'DVX3', 'DIV4', 'DIV5', 'DIV6', 'DIV6', 'MPG3', 'AP41', 'WMV7', 'WMV8']
		, 'lib'	:	'lavc'
		, 'settings' : ':vmax_b_frames=0'
		, 'ext' : 'msmpeg4'
	}
	, 'msmpeg4v2'	: {
		'fourcc':	['MP42', 'DIV2']
		, 'lib'	:	'lavc'
		, 'settings' : ':vmax_b_frames=0'
		, 'ext' : 'msmpeg4v2'
	}
	, 'wmv1'	: {
		'fourcc':	['WMV1']
		, 'lib'	:	'lavc'
		, 'settings' : ':vmax_b_frames=0'
		, 'ext' : 'wmv1'
	}
	, 'wmv2'	: {
		'fourcc':	['WMV2']
		, 'lib'	:	'lavc'
		, 'settings' : ':vmax_b_frames=0'
		, 'ext' : 'wmv2'
	}
	, 'rv10'	: {
		'fourcc':	['RV10', 'RV13']
		, 'lib'	:	'lavc'
		, 'settings' : ':vmax_b_frames=0'
		, 'ext' : 'rv10'
	}
	, 'rv20'	: {
		'fourcc':	['rv20']
		, 'lib'	:	'lavc'
		, 'settings' : ':vmax_b_frames=0'
		, 'ext' : 'rv20'
	}
	, 'libtheora'	: {
		'fourcc':	['LIBT']
		, 'lib'	:	'lavc'
		, 'settings' : ':vmax_b_frames=0'
		, 'ext' : 'theora'
	}
	, 'mpeg1video'	: {
		'fourcc':	['MPEG', 'MPG1']
		, 'lib'	:	'lavc'
		, 'settings' : ':vmax_b_frames=0'
		, 'ext' : 'mp1v'
	}
	, 'mpeg2video'	: {
		'fourcc':	['MPEG', 'MPG2']
		, 'lib'	:	'lavc'
		, 'settings' : ':vmax_b_frames=0'
		, 'ext' : 'mp2v'
	}
	, 'huffyuv'	: {
		'fourcc':	['HFYU']
		, 'lib'	:	'lavc'
		, 'settings' : ':vmax_b_frames=0'
		, 'ext' : 'huffyuv'
	}
	, 'ffvhuff'	: {
		'fourcc':	[]
		, 'lib'	:	'lavc'
		, 'settings' : ':vmax_b_frames=0'
		, 'ext' : 'ffvhuff'
	}
	, 'asv1'	: {
		'fourcc':	[]
		, 'lib'	:	'lavc'
		, 'settings' : ':vmax_b_frames=0'
		, 'ext' : 'asv1'
	}
	, 'asv2'	: {
		'fourcc':	[]
		, 'lib'	:	'lavc'
		, 'settings' : ':vmax_b_frames=0'
		, 'ext' : 'asv2'
	}
	, 'ffv1'	: {
		'fourcc':	['FFV1']
		, 'lib'	:	'lavc'
		, 'settings' : ':vmax_b_frames=0'
		, 'ext' : 'ffv1'
	}
	, 'svq1'	: {
		'fourcc':	['SVQ1','svq1']
		, 'lib'	:	'lavc'
		, 'settings' : ':vmax_b_frames=0'
		, 'ext' : 'svq1'
	}
	, 'flv'		: {
		'fourcc':	['FLV1']
		, 'lib'	:	'lavc'
		, 'settings' : ':vmax_b_frames=0'
		, 'ext' : 'flv'
	}
	, 'flashsv'	: {
		'fourcc':	[]
		, 'lib'	:	'lavc'
		, 'settings' : ':vmax_b_frames=0'
		, 'ext' : 'flashsv'
	}
	, 'dvvideo'	: {
		'fourcc':	[]
		, 'lib'	:	'lavc'
		, 'settings' : ':vmax_b_frames=0'
		, 'ext' : 'dv'
	}
	, 'snow'	: {
		'fourcc':	[]
		, 'lib'	:	'lavc'
		, 'settings' : ':vmax_b_frames=0'
		, 'ext' : 'snow'
	}
	, 'zmbv'	: {
		'fourcc':	[]
		, 'lib'	:	'lavc'
		, 'settings' : ':vmax_b_frames=0'
		, 'ext' : 'zmbv'
	}
	, 'dnxhd'	: {
		'fourcc':	[]
		, 'lib'	:	'lavc'
		, 'settings' : ':vmax_b_frames=0'
		, 'container' : ['mov']
	}
	
}


audio_codecs = {
	'copy'	: {
		'rates'			:	[]
		, 'bitrates'	:	[]
		, 'channels'	:	[]
		, 'd_rate'		:	''
		, 'd_bitrate'	:	''
		, 'd_channels'	:	''
		, 'ext'			:	''
		, 'settings'	:	''
	}
	, 'nosound'	: {
		'rates'			:	[]
		, 'bitrates'	:	[]
		, 'channels'	:	[]
		, 'd_rate'		:	''
		, 'd_bitrate'	:	''
		, 'd_channels'	:	''
		, 'ext'			:	''
		, 'settings'	:	''
	}
	, 'not-set'	: {
		'rates'			:	['11025', '12000', '16000', '22050', '24000', '32000', '44100', '48000']
		, 'bitrates'	:	['8', '16', '24', '32', '40', '48', '56', '64', '72', '80', '88', '96', '104', '112', '120', '128', '136', '144', '152', '160', '168', '176', '184', '192', '200', '208', '216', '224', '232', '240', '248', '256', '264', '272', '280', '288', '296', '304', '312', '320', '328', '336', '344', '352', '360', '368', '376', '384', '392', '400', '408', '416', '424', '432', '440', '448', '456', '464', '472', '480', '488', '496', '504', '512', '520', '528', '536', '544', '552', '560', '568', '576', '584', '592', '600', '608', '616', '624', '632', '640']
		, 'channels'	:	['1', '2', '3', '4', '5', '6', '7', '8']
		, 'd_rate'		:	'44100'
		, 'd_bitrate'	:	''
		, 'd_channels'	:	'2'
		, 'ext'			:	''
		, 'settings'	:	''
	}
	, 'pcm'	: {
		'rates'		:	['11025', '12000', '16000', '22050', '24000', '32000', '44100', '48000']
		, 'bitrates'	:	[]
		, 'channels'	:	['1', '2', '3', '4', '5', '6', '7', '8']
		, 'd_rate'	:	'44100'
		, 'd_bitrate'	:	''
		, 'd_channels'	:	'2'
		, 'ext'		:	'wav'
		, 'settings'	:	''
	}
	, 'mp3lame'	: {
		'rates'			:	['11025', '12000', '16000', '22050', '24000', '32000', '44100', '48000']
		, 'bitrates'	:	['32', '40', '48', '56', '64', '80', '96', '112', '128', '144', '160', '192', '224', '256', '320']
		, 'channels'	:	['mono', 'stereo', 'joint-stereo', 'dualchannel']
		, 'd_rate'		:	'44100'
		, 'd_bitrate'	:	'128'
		, 'd_channels'	:	'joint-stereo'
		, 'ext'			:	'mp3'
		, 'settings'	:	':cbr'
	}
	, 'twolame'	: {
		'rates'			:	['16000', '22050', '24000', '32000', '44100', '48000']
		, 'bitrates'	:	['8', '16', '24', '32', '40', '48', '56', '64', '80', '96', '112', '128', '144', '160', '192', '24', '256', '320', '384']
		, 'channels'	:	['1', '2']
		, 'd_rate'		:	'44100'
		, 'd_bitrate'	:	'128'
		, 'd_channels'	:	'2'
		, 'ext'			:	'mp2'
		, 'settings'	:	''
	}
	, 'toolame'	: {
		'rates'			:	['16000', '22050', '24000', '32000', '44100', '48000']
		, 'bitrates'	:	['8', '16', '24', '32', '40', '48', '56', '64', '80', '96', '112', '128', '144', '160', '192', '24', '256', '320', '384']
		, 'channels'	:	['1', '2']
		, 'd_rate'		:	'44100'
		, 'd_bitrate'	:	'128'
		, 'd_channels'	:	'2'
		, 'ext'			:	'mp2'
		, 'settings'	:	''
	}
	, 'faac'	: {
		'rates'			:	['16000', '22050', '24000', '32000', '44100', '48000']
		, 'bitrates'	:	['8', '16', '24', '32', '40', '48', '56', '64', '72', '80', '88', '96', '104', '112', '120', '128', '136', '144', '152', '160', '168', '176', '184', '192', '200', '208', '216', '224', '232', '240', '248', '256', '264', '272', '280', '288', '296', '304', '312', '320', '328', '336', '344', '352', '360', '368', '376', '384', '392', '400', '408', '416', '424', '432', '440', '448', '456', '464', '472', '480', '488', '496', '504', '512', '520', '528', '536', '544', '552', '560', '568', '576', '584', '592', '600', '608', '616', '624', '632', '640']
		, 'channels'	:	['1', '2', '3', '4', '5', '6']
		, 'd_rate'		:	'44100'
		, 'd_bitrate'	:	'128'
		, 'd_channels'	:	'2'
		, 'ext'			:	'aac'
		, 'settings'	:	':tns'
	}
	#lavc
	, 'ac3'	: {
		'rates'			:	['32000', '44100', '48000']
		, 'bitrates'	:	['8', '16', '24', '32', '40', '48', '56', '64', '72', '80', '88', '96', '104', '112', '120', '128', '136', '144', '152', '160', '168', '176', '184', '192', '200', '208', '216', '224', '232', '240', '248', '256', '264', '272', '280', '288', '296', '304', '312', '320', '328', '336', '344', '352', '360', '368', '376', '384', '392', '400', '408', '416', '424', '432', '440', '448', '456', '464', '472', '480', '488', '496', '504', '512', '520', '528', '536', '544', '552', '560', '568', '576', '584', '592', '600', '608', '616', '624', '632', '640']
		, 'channels'	:	['1', '2', '3', '4', '5', '6']
		, 'd_rate'		:	'44100'
		, 'd_bitrate'	:	'128'
		, 'd_channels'	:	'2'
		, 'ext'			:	'ac3'
		, 'settings'	:	''
	}
	, 'flac'	: {
		'rates'			:	['11025', '12000', '16000', '22050', '24000', '32000', '44100', '48000']
		, 'bitrates'	:	[]
		, 'channels'	:	['1', '2', '3', '4', '5', '6', '7', '8']
		, 'd_rate'		:	'44100'
		, 'd_bitrate'	:	''
		, 'd_channels'	:	'2'
		, 'ext'			:	'flac'
		, 'settings'	:	''
	}
#	, 'g726'	: {
#		'rates'		:	[]
#		, 'bitrates'	:	[]
#		, 'channels'	:	[]
#		, 'd_rate'	:	''
#		, 'd_bitrate'	:	''
#		, 'd_channels'	:	''
#	}
	, 'libfaac'	: {
		'rates'			:	['16000', '22050', '24000', '32000', '44100', '48000']
		, 'bitrates'	:	['8', '16', '24', '32', '40', '48', '56', '64', '72', '80', '88', '96', '104', '112', '120', '128', '136', '144', '152', '160', '168', '176', '184', '192', '200', '208', '216', '224', '232', '240', '248', '256', '264', '272', '280', '288', '296', '304', '312', '320', '328', '336', '344', '352', '360', '368', '376', '384', '392', '400', '408', '416', '424', '432', '440', '448', '456', '464', '472', '480', '488', '496', '504', '512', '520', '528', '536', '544', '552', '560', '568', '576', '584', '592', '600', '608', '616', '624', '632', '640']
		, 'channels'	:	['1', '2', '3', '4', '5', '6', '7', '8']
		, 'd_rate'		:	'44100'
		, 'd_bitrate'	:	'128'
		, 'd_channels'	:	'2'
		, 'ext'			:	'aac'
		, 'settings'	:	''
	}
#	, 'libgsm'	: {
#		'rates'		:	[]
#		, 'bitrates'	:	[]
#		, 'channels'	:	[]
#		, 'd_rate'	:	''
#		, 'd_bitrate'	:	''
#		, 'd_channels'	:	''
#	}
#	, 'libgsm_ms'	: {
#		'rates'		:	[]
#		, 'bitrates'	:	[]
#		, 'channels'	:	[]
#		, 'd_rate'	:	''
#		, 'd_bitrate'	:	''
#		, 'd_channels'	:	''
#	}
#	, 'libmp3lame'	: {
#		'rates'		:	['11025', '12000', '16000', '22050', '24000', '32000', '44100', '48000']
#		, 'bitrates'	:	['32', '40', '48', '56', '64', '80', '96', '112', '128', '144', '160', '192', '224', '256', '320']
#		, 'channels'	:	['mono', 'stereo', 'joint-stereo', 'dualchannel']
#		, 'd_rate'	:	'44100'
#		, 'd_bitrate'	:	'128'
#		, 'd_channels'	:	'joint-stereo'
#	}
	, 'mp2'	: {
		'rates'			:	['16000', '22050', '24000', '32000', '44100', '48000']
		, 'bitrates'	:	['8', '16', '24', '32', '40', '48', '56', '64', '80', '96', '112', '128', '144', '160', '192', '24', '256', '320', '384']
		, 'channels'	:	['2']
		, 'd_rate'		:	'44100'
		, 'd_bitrate'	:	'128'
		, 'd_channels'	:	'2'
		, 'ext'			:	'mp2'
		, 'settings'	:	''
	}
	, 'wmav1'	: {
		'rates'			:	['8000', '11025', '16000', '22050', '44100']
		, 'bitrates'	:	['8', '16', '24', '32', '40', '48', '56', '64', '72', '80', '88', '96', '104', '112', '120', '128', '136', '144', '152', '160', '168', '176', '184', '192', '200', '208', '216', '224', '232', '240', '248', '256', '264', '272', '280', '288', '296', '304', '312', '320', '328', '336', '344', '352', '360', '368', '376', '384', '392', '400', '408', '416', '424', '432', '440', '448', '456', '464', '472', '480', '488', '496', '504', '512', '520', '528', '536', '544', '552', '560', '568', '576', '584', '592', '600', '608', '616', '624', '632', '640']
		, 'channels'	:	['1', '2']
		, 'd_rate'		:	'44100'
		, 'd_bitrate'	:	'128'
		, 'd_channels'	:	'2'
		, 'ext'			:	'wma'
		, 'settings'	:	''
	}	
	, 'wmav2'	: {
		'rates'			:	['8000', '11025', '16000', '22050', '44100']
		, 'bitrates'	:	['8', '16', '24', '32', '40', '48', '56', '64', '72', '80', '88', '96', '104', '112', '120', '128', '136', '144', '152', '160', '168', '176', '184', '192', '200', '208', '216', '224', '232', '240', '248', '256', '264', '272', '280', '288', '296', '304', '312', '320', '328', '336', '344', '352', '360', '368', '376', '384', '392', '400', '408', '416', '424', '432', '440', '448', '456', '464', '472', '480', '488', '496', '504', '512', '520', '528', '536', '544', '552', '560', '568', '576', '584', '592', '600', '608', '616', '624', '632', '640']
		, 'channels'	:	['1', '2']
		, 'd_rate'		:	'44100'
		, 'd_bitrate'	:	'128'
		, 'd_channels'	:	'2'
		, 'ext'			:	'wma'
		, 'settings'	:	''
	}	
#	, 'libvorbis'	: {
#		'rates'			:	['8000', '11025', '16000', '22050', '44100']
#		, 'bitrates'	:	['8', '16', '24', '32', '40', '48', '56', '64', '72', '80', '88', '96', '104', '112', '120', '128', '136', '144', '152', '160', '168', '176', '184', '192', '200', '208', '216', '224', '232', '240', '248', '256', '264', '272', '280', '288', '296', '304', '312', '320', '328', '336', '344', '352', '360', '368', '376', '384', '392', '400', '408', '416', '424', '432', '440', '448', '456', '464', '472', '480', '488', '496', '504', '512', '520', '528', '536', '544', '552', '560', '568', '576', '584', '592', '600', '608', '616', '624', '632', '640']
#		, 'channels'	:	['2']
#		, 'd_rate'		:	'44100'
#		, 'd_bitrate'	:	'128'
#		, 'd_channels'	:	'2'
#		, 'ext'			:	'ogg'
#		, 'settings'	:	''
#	}
	
#	, 'pcm_s32le'
#	, 'pcm_s32be'
#	, 'pcm_u32le'
#	, 'pcm_u32be'
#	, 'pcm_s24le'
#	, 'pcm_s24be'
#	, 'pcm_u24le'
#	, 'pcm_u24be'
#	, 'pcm_s16le'
#	, 'pcm_s16be'
#	, 'pcm_u16le'
#	, 'pcm_u16be'
#	, 'pcm_s8'
#	, 'pcm_u8'
#	, 'pcm_alaw'
#	, 'pcm_mulaw'
#	, 'pcm_s24daud'
#	, 'pcm_zork'
#	, 'roq_dpcm'
#	, 'sonic'
#	, 'sonicls'


}

container_formats = {
	'avi (odml)'	: {
		'format'	: 'avi'
		, 'lib'		: 'avi'
		, 'ext'		: 'avi'
	}
	, 'not-set'	: {
		'format'	: ''
		, 'lib'		: ''
		, 'ext'		: ''
	}
	, 'avi (noodml)' : {
		'format' : 'avi_noodml'
		, 'lib'		: 'avi'
		, 'ext'		: 'avi'
	}
	, 'avi (lavf)' : {
		'format' : 'avi'
		, 'lib'		: 'lavf'
		, 'ext'		: 'avi'
	}
	, 'asf' : {
		'format' : 'asf'
		, 'lib'		: 'lavf'
		, 'ext'		: 'asf'
	}
	, 'flv' : {
		'format' : 'flv'
		, 'lib'		: 'lavf'
		, 'ext'		: 'flv'
	}
	, 'rm' : {
		'format' : 'rm'
		, 'lib'		: 'lavf'
		, 'ext'		: 'rm'
	}
	, 'nut' : {
		'format' : 'nut'
		, 'lib'		: 'lavf'
		, 'ext'		: 'nut'
	}
	, 'mov' : {
		'format' : 'mov'
		, 'lib'		: 'lavf'
		, 'ext'		: 'mov'
	}
	, 'mp4' : {
		'format' : 'mp4'
		, 'lib'		: 'lavf'
		, 'ext'		: 'mp4'
	}
	, 'dv' : {
		'format' : 'dv'
		, 'lib'		: 'lavf'
		, 'ext'		: 'dv'
	}
	, 'matroska' : {
		'format' : 'matroska'
		, 'lib'		: 'lavf'
		, 'ext'		: 'mkv'
	}
	, 'psp' : {
		'format' : 'psp'
		, 'lib'		: 'lavf'
		, 'ext'		: 'mp4'
	}
	, 'ipod' : {
		'format' : 'ipod'
		, 'lib'		: 'lavf'
		, 'ext'		: 'mp4'
	}
	, 'mpeg' : {
		'format' : 'mpeg'
		, 'lib'		: 'mpeg'
		, 'ext'		: 'mpg'
	}
	, 'rawaudio' : {
		'format' : 'rawaudio'
		, 'lib'		: 'rawaudio'
		, 'ext'		: ''
	}
	, 'rawvideo' : {
		'format' : 'rawvideo'
		, 'lib'		: 'rawvideo'
		, 'ext'		: ''
	}
}

resize_methods = {
	'touch_outside' : _('Touch outside')
	, 'keep_aspect' : _('Keep Aspect')
	, 'touch_inside' : _('Touch inside')
	, 'simple' : _('Simple')
	, 'none' : _('not-set')
}

deinterlacers = {
	'disabled' : _("disabled")
	, 'lavcdeint' : _("lavcdeint")
	, 'kerndeint' : _("kerndeint")
	, 'filmdint' : _("filmdint")
	, 'linear_blend' : _("linear blend")
	, 'linear_interpolate' : _("linear interpolate")
	, 'cubic_interpolate' : _("cubic interpolate")
	, 'median_deinterlacer' : _("median deinterlacer")
	, 'fd' : _("fd")
}

rotate_methods = {
	'disabled' : _("disabled")
	, 'r_screen' : _("right")
	, 'l_screen' : _("left")
}

vidinfo_default = {
	'ID_FILE_SUB_FILENAME'		: []
	, 'SUBTITLE_STREAMS'		: []
	, 'AUDIO_STREAMS'		: []
	, 'ID_METADATA'			: []
	, 'ID_LENGTH'			: '0'
	, 'ID_FILENAME'			: ''	
	, 'ID_DEMUXER'			: ''
	, 'ID_VIDEO_CODEC'		: ''
	, 'ID_VIDEO_FORMAT'		: ''
	, 'ID_VIDEO_BITRATE'		: '0'
	, 'ID_VIDEO_WIDTH'		: '0'
	, 'ID_VIDEO_HEIGHT'		: '0'
	, 'ID_VIDEO_FPS'		: '0'
	, 'ID_VIDEO_ASPECT'		: '0.0'
	, 'ID_DEMUXER'			: ''
	, 'ID_AUDIO_CODEC'		: ''
	, 'ID_AUDIO_FORMAT'		: ''
	, 'ID_AUDIO_BITRATE'		: '0'
	, 'ID_AUDIO_RATE'		: '0'
	, 'ID_AUDIO_NCH'		: '0'
}

class vidrop:
	def __init__(self):

		self.pathname = os.path.dirname(sys.argv[0])
		self.abspath = os.path.abspath(self.pathname)
		self.gladefile = self.abspath + "/vidrop.glade"
		self.window1 = gtk.glade.XML(self.gladefile,"window1",APP)
		self.mencoder_path = 'mencoder'
		self.mplayer_path = 'mplayer'
		self.vidinfo = vidinfo_default
		self.output_dir = ''
		if platform.system() == "Windows":
			self.passlogfile = self.abspath + '/vidrop2pass.txt'
		else:
			self.passlogfile = '/tmp/vidrop2pass.txt'
		self.ss = '0'
		self.opening = False
		self.stdout_log = ''
		
		dic = {
			"on_button12_clicked" : self.on_button12_clicked,
			"on_button17_clicked" : self.on_button17_clicked,
			"on_button6_clicked" : self.on_button6_clicked,
			"on_button7_clicked" : self.on_button7_clicked,
			"on_combobox2_changed" : self.on_combobox2_changed,
			"on_combobox3_changed" : self.on_combobox3_changed,
			"on_button13_clicked" : self.on_button13_clicked,
			"on_checkbutton4_toggled" : self.on_checkbutton4_toggled,
			"on_checkbutton6_toggled" : self.on_checkbutton6_toggled,
			"on_button3_clicked" : self.on_button3_clicked,
			"on_button4_clicked" : self.on_button4_clicked,
			"on_button5_clicked" : self.on_button5_clicked,
			"on_button16_clicked" : self.on_button16_clicked,
			"on_button8_clicked" : self.on_button8_clicked,
			"on_button25_clicked" : self.on_button25_clicked,
			"on_combobox1_changed" : self.on_combobox1_changed,
			"on_button2_clicked" : self.on_button2_clicked,
			"on_combobox10_changed" : self.on_combobox10_changed,
			"on_checkbutton5_toggled" : self.on_checkbutton5_toggled,
			"on_button11_clicked" : self.on_button11_clicked,
			"on_checkbutton7_toggled" : self.on_checkbutton7_toggled,
			"on_button22_clicked" : self.on_button22_clicked,
			"on_button10_clicked" : self.on_button10_clicked,
			"on_combobox6_changed" : self.on_combobox6_changed,
			"on_button1_clicked" : self.on_button1_clicked,
			"on_button9_clicked" : self.on_button9_clicked,
			"on_button14_clicked" : self.on_button14_clicked,
			"on_button15_clicked" : self.on_button15_clicked,
			"on_button18_clicked" : self.on_button18_clicked,
			"on_button19_clicked" : self.on_button19_clicked,
			"on_button20_clicked" : self.on_button20_clicked,
			"on_button23_clicked" : self.on_button23_clicked,
			"on_button24_clicked" : self.on_button24_clicked,
			"on_comboboxentry5_changed" : self.on_comboboxentry5_changed,
			"on_button21_clicked" : self.on_button21_clicked,
			#"on_window1_delete_event" : (gtk.main_quit)
			"on_window1_delete_event" : self.on_window1_delete_event
		}
		
		self.settings = {}
		
		self.window1.signal_autoconnect (dic)
		
		
		self.window1.get_widget("label29").set_text(_("Version %s") % APP_VERSION)
		self.combobox_maketext("combobox1")
		self.combobox_maketext("combobox2")
		self.combobox_maketext("combobox3")
		self.combobox_maketext("combobox4")
		self.combobox_maketext("combobox5")
		self.combobox_maketext("combobox6")
		self.combobox_maketext("combobox7")
		self.combobox_maketext("combobox8")
		self.combobox_maketext("combobox9")
		
		
		self.window1.get_widget("combobox1").get_model().set_sort_column_id(0,gtk.SORT_ASCENDING)
		self.window1.get_widget("combobox2").get_model().set_sort_column_id(0,gtk.SORT_ASCENDING)
		self.window1.get_widget("combobox3").get_model().set_sort_column_id(0,gtk.SORT_ASCENDING)
		self.window1.get_widget("combobox6").get_model().set_sort_column_id(0,gtk.SORT_ASCENDING)
		
		
		#self.combobox_maketext("combobox10")

		#list container formats
		self.window1.get_widget("combobox1").get_model().clear()
		for format in container_formats:
			self.window1.get_widget("combobox1").append_text(format)
		self.window1.get_widget("combobox1").set_active(0)
		
		#model = self.window1.get_widget("combobox1").get_model()
		#model1 = gtk.TreeModelSort(model).get_model()
		#self.window1.get_widget("combobox1").set_model(model1)
		
		
		
		#list video codecs
		self.window1.get_widget("combobox2").get_model().clear()
		for codec in video_codecs:
			self.window1.get_widget("combobox2").append_text(codec)
		self.window1.get_widget("combobox2").set_active(0)
		
		#list audio codecs
		self.window1.get_widget("combobox3").get_model().clear()
		for codec in audio_codecs:
			self.window1.get_widget("combobox3").append_text(codec)
		self.window1.get_widget("combobox3").set_active(0)
				
		#list resize_methods
		self.window1.get_widget("combobox6").get_model().clear()
		for method in resize_methods:
			self.window1.get_widget("combobox6").append_text(resize_methods[method])
		self.window1.get_widget("combobox6").set_active(0)
		
		#list rotate methods
		self.window1.get_widget("combobox8").get_model().clear()
		for method in rotate_methods:
			self.window1.get_widget("combobox8").append_text(rotate_methods[method])
		self.window1.get_widget("combobox8").set_active(0)

		#list deinterlacers
		self.window1.get_widget("combobox9").get_model().clear()
		for deinterlacer in deinterlacers:
			self.window1.get_widget("combobox9").append_text(deinterlacers[deinterlacer])
		self.window1.get_widget("combobox9").set_active(0)
		
		#list presets
		liststore = gtk.ListStore(gtk.gdk.Pixbuf, gobject.TYPE_STRING, gobject.TYPE_STRING)
		self.window1.get_widget("combobox10").set_model(liststore)
		cell_icon = gtk.CellRendererPixbuf()
		cell0 = gtk.CellRendererText()
		cell1 = gtk.CellRendererText()
		self.window1.get_widget("combobox10").pack_start(cell_icon, False)
		self.window1.get_widget("combobox10").pack_start(cell0, True)
		self.window1.get_widget("combobox10").pack_start(cell1, True)
		self.window1.get_widget("combobox10").add_attribute(cell_icon, 'pixbuf', 0)  
		self.window1.get_widget("combobox10").add_attribute(cell0, 'text', 1)  
		self.window1.get_widget("combobox10").add_attribute(cell1, 'text', 2)  
		cell1.set_property('visible',False)
		self.list_presets()
		
		#self.combobox_select(self.window1.get_widget("combobox10"),'default.ini', 1)	

		#if (len(sys.argv)>1):
		#	for i in range(1,len(sys.argv)):
		#		self.open_file(sys.argv[i])
		
		parser = OptionParser()
		parser.add_option("-i", "--input-file", action="store", type="string", dest="input_file")
		parser.add_option("-o", "--output-dir", action="store", type="string", dest="output_dir")
		parser.add_option("-p", "--preset", action="store", type="string", dest="preset")
		parser.add_option("-s", "--subtitle", action="store", type="string", dest="subtitle")
		parser.add_option("--convert", action="store_true", dest="convert", default=False)
		parser.add_option("--preview", action="store_true", dest="preview", default=False)
		
		(self.arguments, args) = parser.parse_args()
		
		#print "OPTIONS: ",self.arguments
		self.load_preferences()
		
		if self.arguments.preset:
			self.combobox_select(self.window1.get_widget("combobox10"),self.arguments.preset, 2)
			
		if self.arguments.input_file:
			self.open_file(self.arguments.input_file)
			if self.arguments.output_dir:
				self.output_dir = self.arguments.output_dir
			if self.arguments.subtitle:
				self.window1.get_widget("comboboxentry2").child.set_text(self.arguments.subtitle)
		
		if self.arguments.convert:
			if not self.arguments.input_file:
				print _("ERROR: No input file specified!")
				sys.exit(1)
			else:
				self.on_button7_clicked(None)
		elif self.arguments.preview:
			if not self.arguments.input_file:
				print _("ERROR: No input file specified!")
				sys.exit(1)
			else:
				self.on_button6_clicked(None)			
		else:
			self.window1.get_widget("window1").show()
		
		
		
	def on_window1_delete_event(self, widget, data):
		self.save_preferences()
		gtk.main_quit()
		
	def list_presets(self):

		if platform.system() == "Windows":
			paths = [
				os.path.abspath(os.path.join(self.abspath,'presets'))
			]
		else:
			paths = [
				os.path.abspath(os.path.join(self.abspath,'presets'))
				, os.path.join(os.getenv("HOME"),".local","share","vidrop","presets")
				
			]
		self.window1.get_widget("combobox10").get_model().set_sort_column_id(1,gtk.SORT_ASCENDING)
	
		self.window1.get_widget("combobox10").get_model().clear()
		list_iter = self.window1.get_widget("combobox10").get_model().append()
		icon = None
		self.window1.get_widget("combobox10").get_model().set(list_iter,0, icon, 1,'', 2,'')
					
		for path in paths:
			if os.path.isdir(path):
				presets = os.listdir(path)
				for preset in presets:
					m = re.search('(.*?)\.ini$',preset)
					if m:
						preset_fullname = os.path.abspath(os.path.join(path,preset))
						list_iter = self.window1.get_widget("combobox10").get_model().append()
						
						icon_file = os.path.abspath(os.path.join(path,m.group(1)+'.png'))
						if os.path.isfile(icon_file):
							icon = gtk.image_new_from_file(icon_file).get_pixbuf()
						else:
							icon = self.window1.get_widget("combobox10").render_icon(gtk.STOCK_FILE, gtk.ICON_SIZE_BUTTON)
						self.window1.get_widget("combobox10").get_model().set(list_iter,0, icon, 1,preset, 2,preset_fullname)
		
		#self.window1.get_widget("comboboxentry1").set_active(0)
		
	def get_settings(self):
		self.settings['input_file'] = self.window1.get_widget("entry7").get_text()
		self.settings['input_audiofile'] = self.window1.get_widget("entry5").get_text()
		self.settings['input_dirname'] = os.path.dirname(self.settings['input_file'])
		
		self.settings['auto_add_to_batch'] = self.window1.get_widget("checkbutton1").get_active()
		self.settings['rebuild_index'] = self.window1.get_widget("checkbutton2").get_active()

		self.settings['dvd_location'] = self.window1.get_widget("entry1").get_text()


		s = self.window1.get_widget("comboboxentry5").child.get_text()
		m = re.search('	(\d+) \/ (\d+)$',s)
		if m:
			self.settings['input_file'] = 'dvd://'+m.group(1)
			self.settings['dvd_title'] = m.group(1)
			self.settings['dvd_chapter'] = m.group(2)
		else:
			m = re.search('(\d+) \/ (\d+)-(\d+)$',s)
			if m:
				self.settings['input_file'] = 'dvd://'+m.group(1)
				self.settings['dvd_title'] = m.group(1)
				self.settings['dvd_chapter'] = m.group(2)+'-'+m.group(3)
			else:
				m = re.search('(\d+) - (.*?)$',s)
				if m:
					self.settings['dvd_title'] = None
					self.settings['dvd_chapter'] = m.group(1)
				else:
					self.settings['dvd_title'] = None
					self.settings['dvd_chapter'] = None
				
		self.settings['AUDIO_STREAMS_stream'] = ''
		s = self.window1.get_widget("comboboxentry4").child.get_text()
		m = re.search('(\d+)$',s)
		if m:
			self.settings['AUDIO_STREAMS_stream'] = m.group(1)
			
		self.settings['dvd_subtitle_stream'] = ''
		s = self.window1.get_widget("comboboxentry3").child.get_text()
		m = re.search('(\d+)$',s)
		if m:
			self.settings['dvd_subtitle_stream'] = m.group(1)			
		
		self.settings['dvd_rip_subtitles'] = self.window1.get_widget("checkbutton10").get_active()
		
		self.settings['input_subtitle'] = self.window1.get_widget("comboboxentry2").child.get_text()
		
		#self.settings['output_directory'] = self.window1.get_widget("entry23").get_text()
		#self.settings['output_name'] = self.window1.get_widget("entry2").get_text()
		
		self.settings['output_filename'] = self.window1.get_widget("entry3").get_text()
		
		#self.settings['output_suffix'] = self.window1.get_widget("entry3").get_text()
		self.settings['output_container_format'] = container_formats[self.window1.get_widget("combobox1").get_active_text()]['format']
		self.settings['output_container_lib'] = container_formats[self.window1.get_widget("combobox1").get_active_text()]['lib']
		self.settings['output_container_ext'] = container_formats[self.window1.get_widget("combobox1").get_active_text()]['ext']
			
		self.settings['output_video_codec'] = self.window1.get_widget("combobox2").get_active_text()
		self.settings['output_video_codec_settings'] = self.window1.get_widget("entry11").get_text()
		self.settings['output_audio_codec_settings'] = self.window1.get_widget("entry12").get_text()
		
		
		self.settings['output_fourcc'] = self.window1.get_widget("comboboxentry13").get_active_text()
		
		self.settings['output_video_width'] = str(self.window1.get_widget("spinbutton1").get_value_as_int())
		self.settings['output_video_height'] = str(self.window1.get_widget("spinbutton2").get_value_as_int())
		self.settings['output_video_bitrate'] = str(self.window1.get_widget("spinbutton4").get_value_as_int())
		self.settings['output_video_precision'] = str(self.window1.get_widget("spinbutton5").get_value_as_int())
		self.settings['output_video_fps'] = str(self.window1.get_widget("entry4").get_text())
		
		self.settings['resize_method'] = 'touch_outside'
		n = self.window1.get_widget("combobox6").get_active_text()
		for s in resize_methods:
			if resize_methods[s] == n:
				self.settings['resize_method'] = s
	
		
		self.settings['two_pass'] = self.window1.get_widget("checkbutton9").get_active()
		self.settings['output_audio_codec'] = self.window1.get_widget("combobox3").get_active_text()
		self.settings['output_audio_srate'] = self.window1.get_widget("combobox7").get_active_text()
		self.settings['output_audio_channels'] = self.window1.get_widget("combobox4").get_active_text()
		self.settings['output_audio_bitrate'] = self.window1.get_widget("combobox5").get_active_text()
		self.settings['output_audio_normalize'] = self.window1.get_widget("checkbutton3").get_active()
		self.settings['output_audio_normalize_value'] = str(self.window1.get_widget("spinbutton6").get_value())
		self.settings['output_audio_extrastereo'] = self.window1.get_widget("checkbutton11").get_active()
				
		self.settings['subtitle_font'] = pango.FontDescription(self.window1.get_widget("fontbutton1").get_font_name()).get_family()
		self.settings['subtitle_size'] = str(self.window1.get_widget("spinbutton7").get_value_as_int())
		self.settings['subtitle_encoding'] = self.window1.get_widget("comboboxentry8").get_active_text()
		self.settings['subtitle_position'] = str(self.window1.get_widget("spinbutton8").get_value_as_int())
		self.settings['subtitle_width'] = str(self.window1.get_widget("spinbutton9").get_value_as_int())
		
		self.settings['subtitle_ass'] = self.window1.get_widget("checkbutton7").get_active()
		self.settings['subtitle_antialias'] = self.window1.get_widget("checkbutton12").get_active()
		self.settings['subtitle_color'] = self.window1.get_widget("entry6").get_text()
		self.settings['subtitle_outline_color'] = self.window1.get_widget("entry10").get_text()
		self.settings['additional_mencoder_params'] = self.window1.get_widget("entry9").get_text()
		self.settings['output_video_rotate'] = 'disabled'
		n = self.window1.get_widget("combobox8").get_active_text()
		for s in rotate_methods:
			if rotate_methods[s] == n:
				self.settings['output_video_rotate'] = s		
		
		self.settings['output_video_flip'] = self.window1.get_widget("checkbutton8").get_active()
		self.settings['output_video_pp'] = self.window1.get_widget("checkbutton4").get_active()
		
		self.settings['output_video_pp_gamma'] = self.window1.get_widget("spinbutton16").get_value()
		self.settings['output_video_pp_brightness'] = self.window1.get_widget("spinbutton17").get_value()
		self.settings['output_video_pp_contrast'] = self.window1.get_widget("spinbutton18").get_value()
		self.settings['output_video_pp_saturation'] = self.window1.get_widget("spinbutton19").get_value()
		
		self.settings['output_video_blur'] = self.window1.get_widget("checkbutton6").get_active()
		self.settings['output_video_blur_value'] = self.window1.get_widget("spinbutton3").get_value()
		
		n = self.window1.get_widget("combobox9").get_active_text()
		self.settings['output_video_deinterlace'] = 'disabled'
		for s in deinterlacers:
			if deinterlacers[s] == n:
				self.settings['output_video_deinterlace'] = s
		
		self.settings['time_interval_enabled'] = self.window1.get_widget("checkbutton5").get_active()
		self.settings['time_interval_start'] = 60*60*self.window1.get_widget("spinbutton10").get_value_as_int() +60*self.window1.get_widget("spinbutton11").get_value_as_int() +self.window1.get_widget("spinbutton12").get_value_as_int()
		self.settings['time_interval_length'] = (60*60*self.window1.get_widget("spinbutton13").get_value_as_int() +60*self.window1.get_widget("spinbutton14").get_value_as_int() +self.window1.get_widget("spinbutton15").get_value_as_int()) - (60*60*self.window1.get_widget("spinbutton10").get_value_as_int() +60*self.window1.get_widget("spinbutton11").get_value_as_int() +	self.window1.get_widget("spinbutton12").get_value_as_int())


	def generate_command_line(self, video_pass, preview = False):
		#2pass не работи с h261,ljpeg,mjpeg,svq1,wmv2
		# h261: Valid sizes are 176x144, 352x288
		if not self.settings['input_file']:
			return
		
		input_width = float(self.vidinfo['ID_VIDEO_WIDTH'])
		input_height = float(self.vidinfo['ID_VIDEO_HEIGHT'])
		aspect_ratio = input_width / input_height
		
		cmd = 'mencoder "'+self.settings['input_file']+'" -idx'
		
		if self.settings['rebuild_index']:
			cmd = cmd + ' -forceidx'

		# =========== VIDEO_CODECS ===========		
		video_codec_lib = video_codecs[self.settings['output_video_codec']]['lib']
		if video_codec_lib == 'not-set':
			cmd = cmd + ''
		elif video_codec_lib== 'copy':
			cmd = cmd + ' -ovc copy'
		elif video_codec_lib == 'xvid':
			cmd = cmd + ' -ovc xvid -xvidencopts me_quality='+self.settings['output_video_precision']
			cmd = cmd + ('' if self.settings['output_video_bitrate'] == '0' else ':bitrate='+self.settings['output_video_bitrate']) + self.settings['output_video_codec_settings']
			if self.settings['two_pass']:
				if video_pass ==1:
					cmd = cmd + ':pass=1:turbo'
				else:
					cmd = cmd + ':pass=2'
								
		elif video_codec_lib == 'x264':
			cmd = cmd + ' -ovc x264 -x264encopts subq='+self.settings['output_video_precision']
			cmd = cmd + ('' if self.settings['output_video_bitrate'] == '0' else ':bitrate='+self.settings['output_video_bitrate']) + self.settings['output_video_codec_settings']
			if self.settings['two_pass']:
				if video_pass ==1:
					cmd = cmd + ':pass=1:turbo'
				else:
					cmd = cmd + ':pass=2'		
			
		elif video_codec_lib == 'lavc':
			cmd = cmd + ' -ovc lavc -lavcopts vcodec='+self.settings['output_video_codec']+':vme='+self.settings['output_video_precision']
			cmd = cmd + ('' if self.settings['output_video_bitrate'] == '0' else ':vbitrate='+self.settings['output_video_bitrate']) + self.settings['output_video_codec_settings']
			if self.settings['two_pass']:
				if video_pass ==1:
					cmd = cmd + ':vpass=1:turbo'
				else:
					cmd = cmd + ':vpass=2'	
		# =========== /VIDEO_CODECS ===========		


		if video_codec_lib not in ['copy']:
			# =========== DEINTERLACE ===========		
			if self.settings['output_video_deinterlace'] == 'lavcdeint':
				cmd = cmd + ' -vf lavcdeint'	
			elif self.settings['output_video_deinterlace'] == 'kerndeint':
				cmd = cmd + ' -vf kerndeint'	
			elif self.settings['output_video_deinterlace'] == 'filmdint':
				cmd = cmd + ' -vf filmdint'	
			elif self.settings['output_video_deinterlace'] == 'linear_blend':
				cmd = cmd + ' -vf pp=lb'	
			elif self.settings['output_video_deinterlace'] == 'linear_interpolate':
				cmd = cmd + ' -vf pp=li'	
			elif self.settings['output_video_deinterlace'] == 'cubic_interpolate':
				cmd = cmd + ' -vf pp=ci'	
			elif self.settings['output_video_deinterlace'] == 'median_deinterlacer':
				cmd = cmd + ' -vf pp=md'	
			elif self.settings['output_video_deinterlace'] == 'fd':
				cmd = cmd + ' -vf pp=fd'	
			# =========== /DEINTERLACE ===========		
			
			# =========== RESIZE ===========				
			if self.settings['resize_method'] == 'touch_outside':
				if self.settings['subtitle_ass']:
					cmd = cmd + ' -vf-add scale=-2:'+self.settings['output_video_height']+' -vf-add crop='+self.settings['output_video_width']+':'+self.settings['output_video_height']+' -vf-add expand='+self.settings['output_video_width']+':'+self.settings['output_video_height']+':-1:-1:0,ass'
				else:
					cmd = cmd + ' -vf-add scale=-2:'+self.settings['output_video_height']+' -vf-add crop='+self.settings['output_video_width']+':'+self.settings['output_video_height']+' -vf-add expand='+self.settings['output_video_width']+':'+self.settings['output_video_height']+':-1:-1:1'
			elif self.settings['resize_method'] == 'keep_aspect':
				if self.settings['subtitle_ass']:
					cmd = cmd + ' -vf-add scale='+self.settings['output_video_width']+':'+str(int(round(float(self.settings['output_video_width']) / aspect_ratio )))+',ass'
				else:
					cmd = cmd + ' -vf-add scale='+self.settings['output_video_width']+':'+str(int(round(float(self.settings['output_video_width']) / aspect_ratio )))+',expand='+self.settings['output_video_width']+':'+str(int(round(float(self.settings['output_video_width']) / aspect_ratio )))+':-1:-1:1'
			elif self.settings['resize_method'] == 'touch_inside':
				aspect_ratio1 = float(self.settings['output_video_width']) / float(self.settings['output_video_height'])
				if self.settings['subtitle_ass']:
					if aspect_ratio1 < aspect_ratio:
						cmd = cmd + ' -vf-add expand='+str(int(input_width))+ ':'+str(int(round(input_width/aspect_ratio1)))+':-1:-1:0,scale='+self.settings['output_video_width']+':'+self.settings['output_video_height']+',ass'
					else:
						cmd = cmd + ' -vf-add expand='+str(int(round(input_height*aspect_ratio1)))+':'+str(int(input_height))+':-1:-1:0,scale='+self.settings['output_video_width']+':'+self.settings['output_video_height']+',ass'
				else:
					if aspect_ratio1 < aspect_ratio:
						cmd = cmd + ' -vf-add expand='+str(int(input_width))+ ':'+str(int(round(input_width/aspect_ratio1)))+':-1:-1:0,scale='+self.settings['output_video_width']+':'+self.settings['output_video_height']+',expand=::-1:-1:1'
					else:
						cmd = cmd + ' -vf-add expand='+str(int(round(input_height*aspect_ratio1)))+':'+str(int(input_height))+':-1:-1:0,scale='+self.settings['output_video_width']+':'+self.settings['output_video_height']+',expand=::-1:-1:1'
				
			elif self.settings['resize_method'] == 'simple':
				if self.settings['subtitle_ass']:
					cmd = cmd + ' -vf-add scale='+self.settings['output_video_width']+':'+self.settings['output_video_height']+',ass'
				else:
					cmd = cmd + ' -vf-add scale='+self.settings['output_video_width']+':'+self.settings['output_video_height']+',expand=::-1:-1:1'

				#cmd = cmd + ' -aspect '+self.settings['output_video_width']+'/'+self.settings['output_video_height']
			elif self.settings['resize_method'] == 'none':
				if self.settings['subtitle_ass']:
					cmd = cmd + ' -noautoexpand -vf-add ass'
				else:
					cmd = cmd + ' -vf-add expand=::-1:-1:1'
			# =========== /RESIZE ===========				

			# =========== ROTATE ===========				
			if self.settings['output_video_rotate'] == 'r_screen':
				cmd = cmd + ' -vf-add rotate=1'
				if self.settings['subtitle_ass']:
					cmd = cmd + ' -noautoexpand'
			elif self.settings['output_video_rotate'] == 'l_screen':
				cmd = cmd + ' -vf-add rotate=2'
				if self.settings['subtitle_ass']:
					cmd = cmd + ' -noautoexpand'
			# =========== /ROTATE ===========				
		
			# =========== SUBTITLES ===========				
			if self.settings['input_subtitle'] and self.settings['resize_method']!=5:
				cmd = cmd + ' -sub "'+self.settings['input_subtitle']+'"'
				cmd = cmd + ' -font "'+self.settings['subtitle_font']+'"'
				cmd = cmd + ' -subpos '+self.settings['subtitle_position']
				cmd = cmd + ' -subwidth '+self.settings['subtitle_width']
				cmd = cmd + ' -subfont-text-scale '+self.settings['subtitle_size']
				cmd = cmd + ' -subcp '+self.settings['subtitle_encoding']
				if self.settings['subtitle_ass']:
					cmd = cmd + ' -ass'
					if self.settings['subtitle_antialias']:
						cmd = cmd + ' -ass-hinting 2'
					if self.settings['subtitle_color'] and self.settings['subtitle_outline_color']:
						cmd = cmd + ' -ass-color '+self.settings['subtitle_color']
						cmd = cmd + ' -ass-border-color '+self.settings['subtitle_outline_color']
			# =========== /SUBTITLES ===========				
			
			if self.settings['output_video_flip']:
				cmd = cmd + ' -noautoexpand -vf-add flip'
			
			if self.settings['output_video_pp']:
				cmd = cmd + ' -vf-add eq2='+str(self.settings['output_video_pp_gamma'])+':'+str(self.settings['output_video_pp_contrast'])+':'+str(self.settings['output_video_pp_brightness'])+':'+str(self.settings['output_video_pp_saturation'])+'::::'
		
			if self.settings['output_video_blur']:
				cmd = cmd + ' -vf-add smartblur=5:'+str(self.settings['output_video_blur_value'])+':0'
		
			if self.settings['output_video_fps'] not in ['', '0'] :
				cmd = cmd + ' -ofps ' + str(self.settings['output_video_fps'])
			
			
		# =========== OUTPUT CONTAINER ===========		
		if self.settings['output_container_lib'] == 'lavf':
			cmd = cmd + ' -of lavf -lavfopts format='+self.settings['output_container_format']
		elif self.settings['output_container_lib'] == 'avi':
			if self.settings['output_container_format'] == 'avi':
				cmd = cmd + ' -of avi'
			elif self.settings['output_container_format'] == 'avi_noodml':
				cmd = cmd + ' -of avi -noodml'
		elif self.settings['output_container_lib'] == 'rawaudio':
			cmd = cmd + ' -of rawaudio'
		elif self.settings['output_container_lib'] == 'rawvideo':
			cmd = cmd + ' -of rawvideo'
		elif self.settings['output_container_lib'] == 'mpeg':
			cmd = cmd + ' -of mpeg'
		#elif self.settings['output_container_lib'] == 'MP4Box':
		#	cmd = cmd + ' -of avi'
		# =========== /OUTPUT CONTAINER ===========		

			
		if self.settings['dvd_location']:
			cmd = cmd + ' -dvd-device "'+self.settings['dvd_location']+'"'

		if self.settings['AUDIO_STREAMS_stream']: #this MUST be before audio codec settings
			cmd = cmd + ' -aid '+self.settings['AUDIO_STREAMS_stream']
		
		if self.settings['two_pass'] and video_pass == 1:
			cmd = cmd + ' -nosound'
		else:
			if self.settings['input_audiofile']:
				cmd = cmd + ' -audiofile "'+self.settings['input_audiofile']+'"'
						
			if self.settings['output_audio_codec'] == 'copy':
				cmd = cmd + ' -oac copy'
			elif self.settings['output_audio_codec'] == 'nosound':
				cmd = cmd + ' -nosound'
			elif self.settings['output_audio_codec'] == 'not-set':
				pass
			elif self.settings['output_audio_codec'] == 'pcm':
				cmd = cmd + ' -oac pcm'
			elif self.settings['output_audio_codec'] == 'mp3lame':
				cmd = cmd + ' -oac mp3lame -lameopts br='+self.settings['output_audio_bitrate']+self.settings['output_audio_codec_settings']
				if self.settings['output_audio_channels'] == 'stereo':
					cmd = cmd + ':mode=0'
				elif self.settings['output_audio_channels'] == 'joint-stereo':
					cmd = cmd + ':mode=1'
				elif self.settings['output_audio_channels'] == 'dualchannel':
					cmd = cmd + ':mode=2'
				elif self.settings['output_audio_channels'] == 'mono':
					cmd = cmd + ':mode=3'
			elif self.settings['output_audio_codec'] == 'twolame':
				cmd = cmd + ' -oac twolame -twolameopts br='+self.settings['output_audio_bitrate']+self.settings['output_audio_codec_settings']
				#cmd = cmd + ':mode='+self.settings['output_audio_channels']
			elif self.settings['output_audio_codec'] == 'toolame':
				cmd = cmd + ' -oac toolame -toolameopts br='+self.settings['output_audio_bitrate']+self.settings['output_audio_codec_settings']
				#cmd = cmd + ':mode='+self.settings['output_audio_channels']
			elif self.settings['output_audio_codec'] == 'faac':
				cmd = cmd + ' -oac faac -faacopts br='+self.settings['output_audio_bitrate']+self.settings['output_audio_codec_settings']
			elif self.settings['output_audio_codec'] == 'libfaac':
				cmd = cmd + ' -oac lavc -lavcopts acodec=libfaac:abitrate='+self.settings['output_audio_bitrate']+self.settings['output_audio_codec_settings']
			elif self.settings['output_audio_codec'] == 'ac3':
				cmd = cmd + ' -oac lavc -lavcopts acodec=ac3:abitrate='+self.settings['output_audio_bitrate']+self.settings['output_audio_codec_settings']
			#elif self.settings['output_audio_codec'] == 'flac':
			#	cmd = cmd + ' -oac lavc -lavcopts acodec=flac'#:abitrate='+self.settings['output_audio_bitrate']
			elif self.settings['output_audio_codec'] == 'mp2':
				cmd = cmd + ' -oac lavc -lavcopts acodec=mp2:abitrate='+self.settings['output_audio_bitrate']+self.settings['output_audio_codec_settings']
			elif self.settings['output_audio_codec'] in ['wmav1','wmav2']:
				cmd = cmd + ' -oac lavc -lavcopts acodec='+self.settings['output_audio_codec']+':abitrate='+self.settings['output_audio_bitrate']+self.settings['output_audio_codec_settings']
			elif self.settings['output_audio_codec'] == 'libvorbis':
				cmd = cmd + ' -oac lavc -lavcopts acodec=libvorbis:abitrate='+self.settings['output_audio_bitrate']+self.settings['output_audio_codec_settings']
		
			
			if self.settings['output_audio_codec'] not in ['copy', 'nosound']:
				if self.settings['output_audio_normalize']:
					cmd = cmd + ' -af volnorm=2:'+self.settings['output_audio_normalize_value']
					
				if self.settings['output_audio_extrastereo']:
					cmd = cmd + ' -af extrastereo=2.5'	
					
				if self.settings['output_audio_codec'] not in ['mp3lame']:
					cmd = cmd + ' -af channels='+self.settings['output_audio_channels']
					
				cmd = cmd + ' -srate '+self.settings['output_audio_srate']		

		if self.settings['two_pass']:
			cmd = cmd + ' -passlogfile "%s"' % self.passlogfile
				
		if preview:
			if self.settings['dvd_chapter']!='':
				cmd = cmd + ' -endpos 15'
			else:
				random.seed()
				if video_pass == 1:
					self.ss = str(int(random.randint(0,round(float(self.vidinfo['ID_LENGTH'])))))
					
				cmd = cmd + ' -ss '+self.ss+' -endpos 15'
		elif self.settings['time_interval_enabled']:
			cmd = cmd + ' -ss '+str(self.settings['time_interval_start'])+' -endpos '+str(self.settings['time_interval_length'])
			
		if self.settings['dvd_chapter']:
			cmd = cmd + ' -chapter '+self.settings['dvd_chapter']
			
		
		if self.settings['dvd_subtitle_stream']:
			if self.settings['dvd_rip_subtitles']:
				cmd = cmd + ' -vobsubout "'+os.path.splitext(self.settings['output_filename'])[0]+'" -vobsuboutindex 0 -sid '+self.settings['dvd_subtitle_stream']
			else:
				cmd = cmd + ' -sid '+self.settings['dvd_subtitle_stream']
							
		cmd = cmd + ' -o "' + self.settings['output_filename'] + '"'
		
		if self.settings['additional_mencoder_params']:
			for field in self.settings:
				self.settings['additional_mencoder_params'] = self.settings['additional_mencoder_params'].replace("%"+field+"%",str(self.settings[field]))
			cmd = cmd + ' '+self.settings['additional_mencoder_params']
		
		if video_codec_lib not in ['copy']:
			cmd = cmd + ' -vf-add harddup'
		#print cmd
		return cmd	

	def generate_mplayer_command_line(self):
		self.get_settings()
		if not self.settings['input_file']:
			return
		
		cmd = 'mplayer "'+self.settings['input_file']+'" -idx'
		
		if self.settings['rebuild_index']:
			cmd = cmd + ' -forceidx'
		
		if self.settings['dvd_location']:
			cmd = cmd + ' -dvd-device "'+self.settings['dvd_location']+'"'
		
		if self.settings['input_audiofile']:
			cmd = cmd + ' -audiofile "'+self.settings['input_audiofile']+'"'
		
		if self.settings['dvd_chapter']:
			cmd = cmd + ' -chapter '+self.settings['dvd_chapter']
			
		if self.settings['AUDIO_STREAMS_stream']:
			cmd = cmd + ' -aid '+self.settings['AUDIO_STREAMS_stream']
		
		if self.settings['dvd_subtitle_stream']:
			cmd = cmd + ' -sid '+self.settings['dvd_subtitle_stream']
					
		return cmd
				
	def on_button16_clicked(self,widget):
		return
		dialog = gtk.FileChooserDialog(title=_("Open multiple images"),action=gtk.FILE_CHOOSER_ACTION_OPEN,
			buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,gtk.STOCK_OPEN,gtk.RESPONSE_OK))

		dialog.set_default_response(gtk.RESPONSE_OK)


		filter = gtk.FileFilter()
		filter.set_name(_("Image files"))
		filter.add_mime_type("image/png")
		filter.add_mime_type("image/jpeg")
		filter.add_mime_type("image/tga")
		dialog.add_filter(filter)
		
		filter = gtk.FileFilter()
		filter.set_name(_("All files"))
		filter.add_pattern("*")
		dialog.add_filter(filter)
				
		dialog.set_select_multiple(True)

		response = dialog.run()
		if response == gtk.RESPONSE_OK:
			files = dialog.get_filenames()
			self.window1.get_widget("entry7").set_text('mf://'+(','.join(files)))
			self.window1.get_widget("entry3").set_text(os.path.dirname(files[0]))
		dialog.destroy()
		return
						
	def on_button17_clicked(self,widget):
		dialog = gtk.FileChooserDialog(title=_("Open A/V File"),action=gtk.FILE_CHOOSER_ACTION_OPEN,
			buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,gtk.STOCK_OPEN,gtk.RESPONSE_OK))

		dialog.set_default_response(gtk.RESPONSE_OK)
		dialog.set_current_folder(self.get_input_dir())


		#filter = gtk.FileFilter()
		#filter.set_name(_("Multimedia files"))
		#filter.add_pattern("*.avi")
		#filter.add_pattern("*.psp")
		#filter.add_mime_type("video/*")
		#filter.add_mime_type("audio/*")
		#dialog.add_filter(filter)

		filter = gtk.FileFilter()
		filter.set_name(_("All files"))
		filter.add_pattern("*")
		dialog.add_filter(filter)

		response = dialog.run()
		if response == gtk.RESPONSE_OK:
			self.open_file(dialog.get_filename())
		dialog.destroy()
		return
	
	def open_file(self,filename):
		if filename:
			dvd_location = self.window1.get_widget("entry1").get_text()
			self.window1.get_widget("entry7").set_text(filename)
			output = os.popen('mplayer "'+filename+'" -sub-fuzziness 1 -quiet -identify -frames 0 -dvd-device "'+dvd_location+'"').read()
			self.get_vidinfo(output)
			if filename == 'dvd://':
				
				p = re.compile('ID_DVD_TITLE_(\d+)_LENGTH=(.*)')
				items = p.findall(output)	
				length = 0.0
				title = 1
				chapters = 0
				for item in items:
					if float(item[1]) > length:
						length = float(item[1])
						title = item[0]
			
				if title != 1:
					m = re.search('ID_DVD_TITLE_'+title+'_CHAPTERS=(.*)',output)
					if m:
						chapters = m.group(1)	
					if chapters:
						self.combobox_select(self.window1.get_widget("comboboxentry5"),title+' / 1-'+chapters)
			
			if os.path.os.path.exists(filename):
				output_directory = os.path.dirname(filename)
			elif re.search('dvd://',filename):
				output_directory = os.getenv('HOME') or os.getenv('USERPROFILE')
			elif re.search('mf://',filename):
				output_directory = os.getenv('HOME') or os.getenv('USERPROFILE')
			else:
				output_directory = os.getenv('HOME') or os.getenv('USERPROFILE')
			
			if not self.output_dir:
				self.output_dir = output_directory
			
			name = os.path.join(self.output_dir,self.get_output_name())
			if name:
				self.window1.get_widget("entry3").set_text(name)

			if self.window1.get_widget("checkbutton1").get_active():
				self.on_button3_clicked(None)
			
	def on_window2_delete_event(self,widget,other):
		widget.destroy()
		return

	def on_window3_delete_event(self,widget,other):
		widget.destroy()
		return
		
	def on_button14_clicked(self,widget):	
		webbrowser.open('http://skss.learnfree.eu')
	
	def on_button18_clicked(self,widget):	
		webbrowser.open('mailto:lokiisyourmaster@gmail.com')
	
	def on_button20_clicked(self, widget):
		webbrowser.open('https://www.paypal.com/cgi-bin/webscr?cmd=_donations&business=lokiisyourmaster%40gmail%2ecom&item_name=viDrop%20video%20transcoder&amount=&no_shipping=0&no_note=1&tax=0&currency_code=EUR&lc=BG&bn=PP%2dDonationsBF&charset=UTF%2d8')
		
	def on_button23_clicked(self,widget):	
		webbrowser.open('http://vidrop.learnfree.eu')		
		
	def on_button24_clicked(self,widget):	
		webbrowser.open('http://www.facebook.com/pages/Video-transcoder-viDrop/118888468176842')		
		
	def on_button21_clicked(self, widget):
		dvdlength = float(self.vidinfo['ID_LENGTH'])
		if dvdlength:
			cdsize = self.getText(_('Bitrate calculator'), _('Enter desired output size'), _('Value (MB):'), '700')
			if cdsize != None:
				cdsize = int(cdsize) if cdsize else 0
				if self.settings['output_audio_codec'] == 'pcm':
					abitrate = float(self.settings['output_audio_channels']) * 16 * int(self.settings['output_audio_srate'])
				elif self.settings['output_audio_codec'] == 'copy':
					abitrate = float(self.vidinfo['ID_AUDIO_BITRATE'])/1000
				elif self.settings['output_audio_codec'] == 'not-set':
					abitrate = 0
				elif self.settings['output_audio_codec'] == 'nosound':
					abitrate = 0
				else:
					abitrate = self.settings['output_audio_bitrate']
				vbitrate = ((((float(cdsize)*1024.0*8.0)/1000.0)*1024.0) - (float(abitrate) * dvdlength)) / (dvdlength)
				self.window1.get_widget("spinbutton4").set_value(int(vbitrate))
		else:
			self.msg_error(_('You must open a file/dvd first!'))
		
	def msg_multiline(self, title, text):
		def close(widget):
			ns.window.get_widget("window3").destroy()
		ns = Namespace()
		ns.window = gtk.glade.XML(self.gladefile,"window3" ,APP)
		gtk.glade.bindtextdomain(APP,DIR)
		dic = {
			"on_window3_delete_event" : (close)
			, "on_button1_clicked" :  (close)
		}
		ns.window.signal_autoconnect (dic)	
		ns.window.get_widget("textview1").get_buffer().set_text(text)
		ns.window.get_widget("window3").set_title(title)
		ns.window.get_widget("window3").show()
		
	def on_button19_clicked(self,widget):
		self.get_settings()
		msg = _('Available variables') + '\n\n'
		for field in self.settings:
			msg = msg + "%"+field +"%\n"
		self.msg_multiline(_('Available variables'), msg)
		return
		
	def on_button9_clicked(self,widget):
		if not self.vidinfo['ID_FILENAME']:
			self.msg_error(_("No input file selected!"))
			return
	
		self.window2 = gtk.glade.XML(self.gladefile,"window2" ,APP)
		gtk.glade.bindtextdomain(APP,DIR)
		dic = {
			"on_window2_delete_event" : self.on_window2_delete_event
		}
		self.window2.signal_autoconnect (dic)	
		
		meta = ''
		
		for line in self.vidinfo['ID_METADATA']:
			meta += '<b>'+line[0]+':</b> ' + line[1]+'\n'
		
		if meta:
			meta = '\n\n<span size="large" weight="bold" underline="single">'+_('Metadata')+'</span>\n' + meta
			
		msg = '<u><b>'+_('File information')+'</b></u>\n'+ \
			'<b>'+_('Name:')+'</b> %s\n'+ \
			'<b>'+_('Format:')+'</b> %s\n'+ \
			'<b>'+_('Size:')+'</b> %.2f MB\n'+ \
			'<b>'+_('Length:')+'</b> %s '+_('seconds') + '\n' + \
			'<b>'+_('Detected subtitles:')+'</b>\n'+ \
			'%s\n'+ \
			'\n'+ \
			'<span size="large" weight="bold" underline="single">'+_('Video')+'</span>\n'+ \
			'<b>'+_('Codec:')+'</b> %s (%s)\n'+ \
			'<b>'+_('Resolution:')+'</b> %sx%s\n'+ \
			'<b>'+_('FPS:')+'</b> %s\n'+ \
			'<b>'+_('Bitrate:')+'</b> %s kb/s\n'+ \
			'\n'+ \
			'<span size="large" weight="bold" underline="single">'+_('File information')+'</span>\n'+ \
			'<b>'+_('Codec:')+'</b> %s (%s)\n'+ \
			'<b>'+_('Bitrate:')+'</b> %d kb/s\n'+ \
			'<b>'+_('Channels:')+'</b> %s\n'+ \
			'<b>'+_('Rate:')+'</b> %s Hz'
	
		msg = msg % (
			self.vidinfo['ID_FILENAME'].replace('&','&amp;')
			, self.vidinfo['ID_DEMUXER']
			, os.path.getsize(self.vidinfo['ID_FILENAME'])/1024/1024 if os.path.isfile(self.vidinfo['ID_FILENAME']) else 0.0
			, self.vidinfo['ID_LENGTH']
			, '\n'.join(self.vidinfo['ID_FILE_SUB_FILENAME'])
			, self.vidinfo['ID_VIDEO_CODEC'], self.vidinfo['ID_VIDEO_FORMAT']
			, self.vidinfo['ID_VIDEO_WIDTH'], self.vidinfo['ID_VIDEO_HEIGHT']
			, str(self.vidinfo['ID_VIDEO_FPS'])
			, str(int(self.vidinfo['ID_VIDEO_BITRATE'])/1024)
			, self.vidinfo['ID_AUDIO_CODEC'], self.vidinfo['ID_AUDIO_FORMAT']
			, int(self.vidinfo['ID_AUDIO_BITRATE'])/1024 if self.vidinfo['ID_AUDIO_BITRATE'] != '' else 0
			, (self.vidinfo['ID_AUDIO_NCH'] if self.vidinfo['ID_AUDIO_NCH'] != '' else '0')
			, (self.vidinfo['ID_AUDIO_RATE'] if self.vidinfo['ID_AUDIO_RATE'] != '' else '0')
		)
		
		if meta:
			msg = msg+meta
		
		self.window2.get_widget("label80").set_markup(msg)
		self.window2.get_widget("textview2").get_buffer().set_text(self.vidinfo['RAW_DATA'])

		return
		
		
	def get_output_name(self):
		output_container = self.window1.get_widget("combobox1").get_active_text()
		if not output_container:
			return
		filename = self.window1.get_widget("entry7").get_text()
		output_suffix = '_converted'
		
		if container_formats[output_container]['format'] == 'rawaudio':
			output_audio_codec = self.window1.get_widget("combobox3").get_active_text()
			output_ext = audio_codecs[output_audio_codec]['ext']
		elif container_formats[output_container]['format'] == 'rawvideo':
			output_audio_codec = self.window1.get_widget("combobox2").get_active_text()
			output_ext = 'rawvideo'
		elif container_formats[output_container]['format'] == 'copy':
			output_ext = ''
		else:
			output_ext = container_formats[output_container]['ext']	
			
		if output_ext:
			return os.path.splitext(os.path.basename(filename))[0] + output_suffix +'.'+ output_ext
		else:
			return os.path.splitext(os.path.basename(filename))[0] + output_suffix
			
	def get_vidinfo(self, output):
		self.opening = True

		self.vidinfo = vidinfo_default
		self.vidinfo['ID_METADATA'] = []
		
		p = re.compile('ID_FILE_SUB_FILENAME=(.*)')
		self.vidinfo['ID_FILE_SUB_FILENAME'] = p.findall(output)
		self.window1.get_widget("comboboxentry2").get_model().clear()
		for s in self.vidinfo['ID_FILE_SUB_FILENAME']:
			self.window1.get_widget("comboboxentry2").append_text(s)
		self.window1.get_widget("comboboxentry2").set_active(0)


		p = re.compile('ID_CLIP_INFO_NAME(\d+)=(.*)')
		names = p.findall(output)	
		for name in names:
				p = re.compile('ID_CLIP_INFO_NAME(\d+)=(.*)')
				m = re.search('ID_CLIP_INFO_VALUE'+name[0]+'=(.*)',output)
				if m:
					self.vidinfo['ID_METADATA'].append([name[1],m.group(1)])
		for s in ['ID_LENGTH', 'ID_FILENAME', 'ID_DEMUXER', 'ID_VIDEO_CODEC', 'ID_VIDEO_FORMAT', 'ID_VIDEO_BITRATE', 'ID_VIDEO_WIDTH', 'ID_VIDEO_HEIGHT', 'ID_VIDEO_FPS', 'ID_VIDEO_ASPECT', 'ID_DEMUXER', 'ID_AUDIO_CODEC', 'ID_AUDIO_FORMAT', 'ID_AUDIO_BITRATE', 'ID_AUDIO_RATE', 'ID_AUDIO_NCH']:
			p = re.compile(s+'=(.*)')
			m = p.findall(output)
			self.vidinfo[s] = m[-1] if m else ''		
			
			
		# audio streams
		self.vidinfo['AUDIO_STREAMS'] = []
		p = re.compile('ID_AID_(\d+)_LANG=(.*)') #DVD languages
		audio_streams = p.findall(output)	
		for s in audio_streams:
			self.vidinfo['AUDIO_STREAMS'].append(s)
		
		p = re.compile('ID_AID_(\d+)_NAME=(.*)') #mkv streams
		audio_streams = p.findall(output)	
		for s in audio_streams:
			self.vidinfo['AUDIO_STREAMS'].append(s)
		self.window1.get_widget("comboboxentry4").get_model().clear()
		self.window1.get_widget("comboboxentry4").append_text('')
		for s in self.vidinfo['AUDIO_STREAMS']:
			self.window1.get_widget("comboboxentry4").append_text('%s - %s' % (s[1], s[0]))
		self.window1.get_widget("comboboxentry4").set_active(0)				
		


		p = re.compile('ID_SID_(\d+)_LANG=(.*)')
		self.vidinfo['SUBTITLE_STREAMS'] = p.findall(output)	
		
		self.window1.get_widget("comboboxentry3").get_model().clear()
		self.window1.get_widget("comboboxentry3").append_text('')
		for s in self.vidinfo['SUBTITLE_STREAMS']:
			self.window1.get_widget("comboboxentry3").append_text('%s - %s' % (s[1], s[0]))
		self.window1.get_widget("comboboxentry3").child.set_text('')
		self.window1.get_widget("comboboxentry3").set_active(0)		
					
		m = re.search('ID_LENGTH=(.*)',output)
		if m:
			self.vidinfo['ID_LENGTH'] = m.group(1)	
		
		self.vidinfo['DVD_CHAPTERS'] = []
		self.window1.get_widget("comboboxentry5").get_model().clear()
		self.window1.get_widget("comboboxentry5").append_text('')
		self.window1.get_widget("comboboxentry5").child.set_text('')
		p = re.compile('ID_DVD_TITLE_(\d*)_CHAPTERS=(.*)')
		chapters = p.findall(output)
		for chapter in chapters:
			self.vidinfo['DVD_CHAPTERS'].append(chapter)
		for chapter in chapters:
			self.window1.get_widget("comboboxentry5").append_text('%s / %d-%s' % (chapter[0], 1, chapter[1]))
			for c in range(0, int(chapter[1])):
				self.window1.get_widget("comboboxentry5").append_text('	%s / %d' % (chapter[0], c+1))

		p = re.compile('ID_CHAPTER_(\d+)_NAME=(.*)')
		chapters = p.findall(output)
		for chapter in chapters:
			self.vidinfo['DVD_CHAPTERS'].append(chapter)
			self.window1.get_widget("comboboxentry5").append_text('%s - %s' % (chapter[0], chapter[1]))
			
				
				
		self.vidinfo['RAW_DATA'] = output
		self.opening = False
			
	def on_button8_clicked(self, widget):
		self.open_file('dvd://')
		
	def run_cmd(self,cmd):
		if platform.system() == "Windows":
			os.system(cmd)
		else:
			os.system('xterm -e '+cmd)	
		
	def run_mplayer(self, filename):
		self.run_cmd('mplayer "'+filename+'"')
		
	def run_mencoder(self, cmdline, current, total):
		if not cmdline:
			return 1
		ns = Namespace()
		
		def cancel_process(widget, other = None):
			ns.kill_process = True
			return True
	
			
		window = gtk.glade.XML(self.gladefile,"window4" ,APP)
		gtk.glade.bindtextdomain(APP,DIR)
		dic = {
			"on_button1_clicked" : (cancel_process),
			"on_window4_delete_event" : (cancel_process)
		}
		window.signal_autoconnect (dic)
		window.get_widget("window4").show()
		while gtk.events_pending():
			gtk.main_iteration_do()
		
		argv =  shlex.split(cmdline)
		info = None
		if platform.system() == "Windows":
			info = asubprocess.STARTUPINFO()
			info.dwFlags |= asubprocess.STARTF_USESHOWWINDOW
			info.wShowWindow = asubprocess.SW_HIDE

		ns.process = asubprocess.Popen(argv, stdout=asubprocess.PIPE, stderr=asubprocess.STDOUT, startupinfo=info)
		
		ns.kill_process = False
		
		window.get_widget("label12").set_text(_("%(current)d of %(total)d") % {'current':current, 'total':total})
		
		i = self.window1.get_widget("combobox10").get_active_iter()
		if i:
			preset_name = self.window1.get_widget("combobox10").get_model().get_value(i,1)
		else:
			preset_name = _('Default')
		window.get_widget("label15").set_text(preset_name)

		while gtk.events_pending():
			gtk.main_iteration_do()
		
		self.stdout_log += cmdline+'\n\n'
				
		while ns.process.poll() == None:
			if ns.kill_process:
				if platform.system() == "Windows":
					os.system("taskkill /PID %d /F" % ns.process.pid) 
				else:
					os.kill(ns.process.pid, signal.SIGKILL)
				ns.process.wait()
				window.get_widget("window4").destroy()
				return ns.process.returncode
			else:
				while gtk.events_pending():
					gtk.main_iteration_do()
				line = ns.process.asyncread(0.1)
				self.stdout_log += line+'\n'
				m = re.search('Pos:\s*(.*?)s\s*(\d+)f\s*\(\s*(\d+)%\)\s*(\d*\.?\d*)fps\s*Trem:\s*(\d+(min))\s*\s*(\d+(mb|kb|gb))',line)
				if m:
					window.get_widget("progressbar1").set_fraction(float(float(m.group(3))/100.0))
					window.get_widget("label7").set_text(_("%(pos)s sec. (%(frame)s frame)") % {'pos':m.group(1), 'frame':m.group(2)})
					window.get_widget("label8").set_text(_("%sfps") % m.group(4))
					window.get_widget("label9").set_text(m.group(5))
					window.get_widget("label10").set_text(m.group(7))
					window.get_widget("label11").set_markup(_('<b>Converting...</b>'))
				else:
					m = re.search('(\d+)\s*%', line)
					if m:
						window.get_widget("progressbar1").set_fraction(float(m.group(1))/100.0)					
						window.get_widget("label11").set_markup(_('<b>Rebuilding index...</b>'))
		
		window.get_widget("window4").destroy()
		while gtk.events_pending():
			gtk.main_iteration_do()
		return ns.process.returncode
		

				
	def on_button6_clicked(self, widget):
		self.get_settings()
		cmd = self.generate_command_line(1, True)

		if cmd:
			self.stdout_log = ''
			self.window1.get_widget("window1").hide()
			while gtk.events_pending():
				gtk.main_iteration_do()	
			if self.settings['two_pass']:
				if self.run_mencoder(cmd,1,2) == 0:
					cmd = self.generate_command_line(2, True)
					if self.run_mencoder(cmd,2,2) == 0:
						#if self.settings['output_container_lib'] == 'MP4Box':
						#	if self.createMP4() == 0:
						#		self.run_mplayer(self.settings['output_filename'])
						#else:
						self.run_mplayer(self.settings['output_filename'])
			else:
				if self.run_mencoder(cmd,1,1) == 0:
					#if self.settings['output_container_lib'] == 'MP4Box':
					#	if self.createMP4(self.settings['output_filename']) == 0:
					#		self.run_mplayer(self.settings['output_filename'])
					#else:
					self.run_mplayer(self.settings['output_filename'])		

			if not self.arguments.preview:
				self.window1.get_widget("window1").show()
				if self.window1.get_widget("checkbutton13").get_active():
					self.msg_multiline(_('mencoder log'),self.stdout_log)
			else:
				sys.exit()
	
	def createMP4(self, filename):
		tmpname = os.path.splitext(filename)[0]+'.tmp'
		output_filename = os.path.splitext(filename)[0]+'.mp4'
		
		os.rename(filename, tmpname)
		output_directory = os.path.dirname(filename)
		
		audio_ext = audio_codecs[self.settings['output_audio_codec']]['ext']
		video_ext = video_codecs[self.settings['output_video_codec']]['ext']
		
		if audio_ext != '':
			audio_ext = '.'+audio_ext
			
		if video_ext != '':
			video_ext = '.'+video_ext
			
			
		self.run_cmd_feedback([
			  [_('Creating MP4 File: extracting raw video stream'), 'mplayer "%s" -dumpvideo -dumpfile "%s/video%s"' % (tmpname, output_directory, video_ext)]
			, [_('Creating MP4 File: extracting raw audio stream'), 'mplayer "%s" -dumpaudio -dumpfile "%s/audio%s"' % (tmpname, output_directory, audio_ext)]
			, [_('Creating MP4 File: joining audio stream'), 'mp4creator -create="%s/audio%s" "%s"' % (output_directory, audio_ext, output_filename)]
			, [_('Creating MP4 File: joining video stream'), 'mp4creator -create="%s/video%s" -rate=%s "%s"' % (output_directory, video_ext, self.settings['output_video_fps'], output_filename)]
		])
		return 0
		

	def run_cmd_feedback(self, cmdlist):
		for row in cmdlist:
			name = row[0]
			cmdline = row[1]
			print "RUNNING: ", name
			print cmdline
			ns = Namespace()
			def cancel_process(widget, other = None):
				ns.kill_process = True
				return True
			
			window = gtk.glade.XML(self.gladefile,"window4" ,APP)
			gtk.glade.bindtextdomain(APP,DIR)
			dic = {
				"on_button1_clicked" : (cancel_process),
				"on_window4_delete_event" : (cancel_process)
			}
			window.signal_autoconnect (dic)
			window.get_widget("window4").show()
			while gtk.events_pending():
				gtk.main_iteration_do()
		
			argv =  shlex.split(cmdline)
			info = None
			if platform.system() == "Windows":
				info = asubprocess.STARTUPINFO()
				info.dwFlags |= asubprocess.STARTF_USESHOWWINDOW
				info.wShowWindow = asubprocess.SW_HIDE

			ns.process = asubprocess.Popen(argv, stdout=asubprocess.PIPE, stderr=asubprocess.STDOUT, startupinfo=info)
		
			ns.kill_process = False
			
			window.get_widget("table1").hide()					
			window.get_widget("progressbar1").set_text(name)					

			while gtk.events_pending():
				gtk.main_iteration_do()
		
			self.stdout_log += cmdline+'\n\n'
				
			while ns.process.poll() == None:
				if ns.kill_process:
					if platform.system() == "Windows":
						os.system("taskkill /PID %d /F" % ns.process.pid) 
					else:
						os.kill(ns.process.pid, signal.SIGKILL)
					ns.process.wait()
					window.get_widget("window4").destroy()
					return ns.process.returncode
				else:
					while gtk.events_pending():
						gtk.main_iteration_do()
					line = ns.process.asyncread(0.1)
					self.stdout_log += line+'\n'

					window.get_widget("progressbar1").pulse()					

		
			window.get_widget("window4").destroy()
			while gtk.events_pending():
				gtk.main_iteration_do()
			if ns.process.returncode != 0:
				return ns.process.returncode
		
		
		
	def on_button7_clicked(self, widget):
		self.get_settings()
		cmd = self.generate_command_line(1)
		if cmd:
			self.stdout_log = ''
			self.window1.get_widget("window1").hide()
			while gtk.events_pending():
				gtk.main_iteration_do()
				
				
			if self.settings['two_pass']:
				result = self.run_mencoder(cmd,1,2)
				if result == 0:
					cmd = self.generate_command_line(2)
					result = self.run_mencoder(cmd,2,2)			
					#if result == 0 and self.settings['output_container_lib'] == 'MP4Box':
					#	self.createMP4(self.settings['output_filename'])
			else:
				result = self.run_mencoder(cmd,1,1)
				#if result == 0 and self.settings['output_container_lib'] == 'MP4Box':
				#	self.createMP4(self.settings['output_filename'])
						
			if not self.arguments.convert:
				self.window1.get_widget("window1").show()
				if self.window1.get_widget("checkbutton13").get_active():
					self.msg_multiline(_('mencoder log'),self.stdout_log)
			else:
				sys.exit()
		
				
	def on_button11_clicked(self,widget):
		dialog = gtk.FileChooserDialog(title=_("Open external audio file"),action=gtk.FILE_CHOOSER_ACTION_OPEN,
			buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,gtk.STOCK_OPEN,gtk.RESPONSE_OK))

		dialog.set_default_response(gtk.RESPONSE_OK)
		dialog.set_current_folder(self.get_input_dir())
		
		filter = gtk.FileFilter()
		filter.set_name("Multimedia files")
		filter.add_mime_type("video/*")
		filter.add_mime_type("audio/*")
		dialog.add_filter(filter)

		filter = gtk.FileFilter()
		filter.set_name(_("All files"))
		filter.add_pattern("*")
		dialog.add_filter(filter)

		response = dialog.run()
		if response == gtk.RESPONSE_OK:
			self.window1.get_widget("entry5").set_text(dialog.get_filename())
		dialog.destroy()
		return
	
	def on_button22_clicked(self,widget):
		dialog = gtk.FileChooserDialog(title=_("Select output filename"),action=gtk.FILE_CHOOSER_ACTION_SAVE,
			buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,gtk.STOCK_SAVE,gtk.RESPONSE_OK))
		dialog.set_default_response(gtk.RESPONSE_OK)
		#dialog.set_current_folder(os.path.dirname(self.window1.get_widget("entry3").get_text()))
		dialog.set_filename((self.window1.get_widget("entry3").get_text()))
		dialog.select_filename((self.window1.get_widget("entry3").get_text()))
		response = dialog.run()
		if response == gtk.RESPONSE_OK:
			self.window1.get_widget("entry3").set_text(dialog.get_filename())
		dialog.destroy()
		return
		
	def get_input_dir(self):
		input_file = self.window1.get_widget("entry7").get_text()
		if input_file:
			return os.path.dirname(input_file)
		else:
			return os.getenv('HOME') or os.getenv('USERPROFILE')
		
	def on_button12_clicked(self,widget):
		dialog = gtk.FileChooserDialog(title=_("Open Subtitle"),action=gtk.FILE_CHOOSER_ACTION_OPEN,
			buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,gtk.STOCK_OPEN,gtk.RESPONSE_OK))

		dialog.set_default_response(gtk.RESPONSE_OK)
		dialog.set_current_folder(self.get_input_dir())
		
		filter = gtk.FileFilter()
		filter.set_name(_("Subtitle files"))
		filter.add_mime_type("text/*")
		filter.add_pattern("*.idx")
		dialog.add_filter(filter)

		filter = gtk.FileFilter()
		filter.set_name(_("All files"))
		filter.add_pattern("*")
		dialog.add_filter(filter)

		response = dialog.run()
		if response == gtk.RESPONSE_OK:
			self.window1.get_widget("comboboxentry2").child.set_text(dialog.get_filename())
		dialog.destroy()
		return
		
	def on_combobox2_changed(self,widget):
		codec = self.window1.get_widget("combobox2").get_active_text()
		self.window1.get_widget("comboboxentry13").child.set_text('')
		
		self.window1.get_widget("comboboxentry13").get_model().clear()
		for fourcc in video_codecs[codec]['fourcc']:
			self.window1.get_widget("comboboxentry13").append_text(fourcc)
		#self.window1.get_widget("comboboxentry13").set_active(0)
		
		widgets = [
			'label22','label23','label24','label25','label26'
			,'label27','label51','checkbutton9','combobox6','spinbutton5'
			,'entry4','spinbutton4','spinbutton2','spinbutton1','comboboxentry13'
			,'frame9','frame11','frame12','label20','entry11'
		]
		if codec == 'copy':
			for w in widgets:
				self.window1.get_widget(w).set_sensitive(False)
		else:
			for w in widgets:
				self.window1.get_widget(w).set_sensitive(True)
		
		if container_formats[self.window1.get_widget("combobox1").get_active_text()]['format'] == 'rawvideo':
		
			if not self.output_dir:
				self.output_dir = os.path.dirname(self.window1.get_widget("entry3").get_text())
			name = os.path.join(self.output_dir,self.get_output_name())
			if name:
				self.window1.get_widget("entry3").set_text(name)
				
		self.get_settings()
		self.window1.get_widget("entry11").set_text(video_codecs[self.settings['output_video_codec']]['settings'])
		
		return
				
	def on_combobox3_changed(self,widget):

		codec = self.window1.get_widget("combobox3").get_active_text()
		
		self.window1.get_widget("combobox4").get_model().clear()
		self.window1.get_widget("combobox5").get_model().clear()
		self.window1.get_widget("combobox7").get_model().clear()
		
		sel = 0
		i = 0
		for rate in audio_codecs[codec]['rates']:
			self.window1.get_widget("combobox7").append_text(rate)
			if rate == audio_codecs[codec]['d_rate']:
				sel = i
			i = i + 1
		self.window1.get_widget("combobox7").set_active(sel)
		
		sel = 0
		i = 0
		for channels in audio_codecs[codec]['channels']:
			self.window1.get_widget("combobox4").append_text(channels)
			if channels == audio_codecs[codec]['d_channels']:
				sel = i
			i = i + 1
		self.window1.get_widget("combobox4").set_active(sel)
		
		sel = 0
		i = 0
		for bitrate in audio_codecs[codec]['bitrates']:
			self.window1.get_widget("combobox5").append_text(bitrate)
			if bitrate == audio_codecs[codec]['d_bitrate']:
				sel = i
			i = i + 1
			
		self.window1.get_widget("combobox5").set_active(sel)
		
		if container_formats[self.window1.get_widget("combobox1").get_active_text()]['format'] == 'rawaudio':
			if not self.output_dir:
				self.output_dir = os.path.dirname(self.window1.get_widget("entry3").get_text())
			name = os.path.join(self.output_dir,self.get_output_name())
			if name:
				self.window1.get_widget("entry3").set_text(name)
				
		widgets = [
			'label34','label31','label32','label32','spinbutton6','checkbutton11'
			,'combobox4','combobox5','combobox7', 'checkbutton3','label49', 'entry12'
		]
		if codec in ['copy','nosound']:
			for w in widgets:
				self.window1.get_widget(w).set_sensitive(False)
		else:
			for w in widgets:
				self.window1.get_widget(w).set_sensitive(True)
		self.get_settings()
		self.window1.get_widget("entry12").set_text(audio_codecs[self.settings['output_audio_codec']]['settings'])
		return
		
	def on_button13_clicked(self, widget):
		self.window1.get_widget("spinbutton16").set_value(1)
		self.window1.get_widget("spinbutton17").set_value(0)
		self.window1.get_widget("spinbutton18").set_value(1)
		self.window1.get_widget("spinbutton19").set_value(1)
		
	def on_checkbutton4_toggled(self,widget):
		self.window1.get_widget("table8").set_sensitive(self.window1.get_widget("checkbutton4").get_active())
		
	def on_checkbutton6_toggled(self,widget):
		self.window1.get_widget("spinbutton3").set_sensitive(self.window1.get_widget("checkbutton6").get_active())		
	
	def on_button15_clicked(self, widget):
		self.window1.get_widget("entry1").set_text('E:\\' if platform.system() == "Windows" else '/dev/dvd')
	
	def on_button25_clicked(self, widget):
		dialog = gtk.FileChooserDialog(title=_("Select DVD location"),action=gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
			buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,gtk.STOCK_OPEN,gtk.RESPONSE_OK))

		dialog.set_default_response(gtk.RESPONSE_OK)

		filter = gtk.FileFilter()
		filter.set_name(_("All files"))
		filter.add_pattern("*")
		dialog.add_filter(filter)

		response = dialog.run()
		if response == gtk.RESPONSE_OK:
			self.window1.get_widget("entry1").set_text(dialog.get_filename())
			self.open_file('dvd://')
		dialog.destroy()
		
		return
		
	def on_combobox1_changed(self, widget):
		if not self.output_dir:
			self.output_dir = os.path.dirname(self.window1.get_widget("entry3").get_text())
		name = os.path.join(self.output_dir, self.get_output_name())
		if name:
			self.window1.get_widget("entry3").set_text(name)
		try:
			if container_formats[self.window1.get_widget("combobox1").get_active_text()]['format']=='rawaudio':
				self.combobox_select(self.window1.get_widget("combobox2"),'copy')
				self.window1.get_widget("frame7").set_sensitive(False)
			else:
				self.window1.get_widget("frame7").set_sensitive(True)
				
			if container_formats[self.window1.get_widget("combobox1").get_active_text()]['format']=='rawvideo':
				self.combobox_select(self.window1.get_widget("combobox3"),'copy')
				self.window1.get_widget("frame8").set_sensitive(False)
			else:
				self.window1.get_widget("frame8").set_sensitive(True)				
				
		except:
			pass
		return
	
	def on_button2_clicked(self, widget):
		i = self.window1.get_widget("combobox10").get_active_iter()
		if i:
			name = self.window1.get_widget("combobox10").get_model().get_value(i,1)
		else:
			name = ''
	
		preset_name = self.getText(_("Save preset"),_("Enter preset name"), _("Name:"), name)
		if preset_name:
			if not re.search('\.ini$',preset_name):
				preset_name = preset_name+'.ini'
			
			if platform.system() == "Windows":
				preset_fullname = os.path.abspath(os.path.join(os.path.dirname(sys.argv[0]),'presets',preset_name))
				result = self.save_preset(preset_fullname)
			else:
				preset_fullname = os.path.abspath(os.path.join(os.path.dirname(sys.argv[0]),'presets',preset_name))
				if self.save_preset(preset_fullname):
					result = True
				else:
					preset_fullname = os.path.join(os.getenv("HOME"),".local","share","vidrop","presets",preset_name)
					result = self.save_preset(preset_fullname)
			if result:			
				self.list_presets()
				self.combobox_select(self.window1.get_widget("combobox10"),preset_fullname, 2)
			else:
				self.msg_error(_("Could not save the preset!"))
		else:
			self.msg_error(_("You must specify preset name!"))
		return

	def msg_error(self,msg):
		dialog = gtk.MessageDialog(self.window1.get_widget("window1"),gtk.DIALOG_MODAL,gtk.MESSAGE_ERROR,gtk.BUTTONS_OK)
		dialog.set_markup(msg)
		dialog.set_position(gtk.WIN_POS_CENTER_ALWAYS)
		dialog.run()
		dialog.destroy()
				
	def msg_info(self,msg):
		dialog = gtk.MessageDialog(self.window1.get_widget("window1"),gtk.DIALOG_MODAL,gtk.MESSAGE_INFO,gtk.BUTTONS_OK)
		dialog.set_markup(msg)
		dialog.set_position(gtk.WIN_POS_CENTER_ALWAYS)
		dialog.run()
		dialog.destroy()		
	
	def msg_confirm(self,msg):
		dialog = gtk.MessageDialog(
			self.window1.get_widget("window1")
			, gtk.DIALOG_DESTROY_WITH_PARENT
			, gtk.MESSAGE_QUESTION
			, gtk.BUTTONS_OK_CANCEL
			, msg
		)
		dialog.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
		response = dialog.run()
		dialog.destroy()

		if response == gtk.RESPONSE_OK:
			return True
		else:
			return False

	def on_button3_clicked(self, widget):
		self.get_settings()
		cmd = self.generate_command_line(1)
		if cmd:
			
			tbuffer = self.window1.get_widget("textview1").get_buffer()
			ei = tbuffer.get_end_iter()
			tbuffer.insert(ei,cmd+'\n')			
			if self.settings['two_pass']:
				cmd = self.generate_command_line(2)
				ei = tbuffer.get_end_iter()
				tbuffer.insert(ei,cmd+'\n')
		return
				
	def on_button4_clicked(self, widget):			
		self.window1.get_widget("textview1").get_buffer().set_text('')
				
	def on_button5_clicked(self, widget):			
		startiter = self.window1.get_widget("textview1").get_buffer().get_start_iter()
		enditer = self.window1.get_widget("textview1").get_buffer().get_end_iter()
		batch = self.window1.get_widget("textview1").get_buffer().get_text(startiter, enditer, True)
		
		batch = batch.split('\n')
		self.stdout_log = ''
		commands = []
		for cmd in batch:
			if cmd!='':
				commands.append(cmd)
		current = 1
		total = len(commands)
		if total > 0:
			self.window1.get_widget("window1").hide()
			while gtk.events_pending():
				gtk.main_iteration_do()
				
			for cmd in commands:
				self.run_mencoder(cmd,current,total)
				current = current + 1
			self.window1.get_widget("window1").show()		
			if self.window1.get_widget("checkbutton13").get_active():
				self.msg_multiline(_('mencoder log'),self.stdout_log)
			

	def on_button10_clicked(self, widget):
		cmd = self.generate_mplayer_command_line()
		if cmd:
			self.window1.get_widget("window1").hide()
			while gtk.events_pending():
				gtk.main_iteration_do()
			#self.run_mplayer(cmd)
			self.run_cmd(cmd)
			self.window1.get_widget("window1").show()		

		
	def on_combobox10_changed(self, widget):
		i = self.window1.get_widget("combobox10").get_active_iter()
		if i:
			preset_fullname = self.window1.get_widget("combobox10").get_model().get_value(i,2)
			if preset_fullname:
				self.load_preset(preset_fullname)
				#if os.stat(preset_fullname).st_uid == os.getuid():
				#	self.window1.get_widget("button1").set_sensitive(True)
				#else:
				#	self.window1.get_widget("button1").set_sensitive(False)
			else:
				self.load_preset('')
					
	def on_combobox6_changed(self, widget):
		n = self.window1.get_widget("combobox6").get_active_text()
		resize_method = 'touch_outside'
		for s in resize_methods:
			if resize_methods[s] == n:
				resize_method = s
		
		if resize_method == 'keep_aspect':
			self.window1.get_widget("spinbutton2").set_sensitive(False)
		else:
			self.window1.get_widget("spinbutton2").set_sensitive(True)
		
	def on_comboboxentry5_changed(self, widget):
		return
				
	def on_button1_clicked(self, widget):
		if not self.msg_confirm(_("Are you sure you want to delete this preset?")):
			return
		
		i = self.window1.get_widget("combobox10").get_active_iter()
		if i:
			preset_fullname = self.window1.get_widget("combobox10").get_model().get_value(i,2)
			try:
				os.unlink(preset_fullname)
			except:
				self.msg_error(_("Can't delete the selected preset. Probably you dont have rights to delete it."))
				return
		else:
			self.msg_error(_("There is no preset selected. Nothing to delete..."))
			return
		self.list_presets()
		self.combobox_select(self.window1.get_widget("combobox10"),'', 2)	
		return
		
		
	def on_checkbutton5_toggled(self, widget):
		self.window1.get_widget("hbox6").set_sensitive(self.window1.get_widget("checkbutton5").get_active())
		
	def on_checkbutton7_toggled(self, widget):
		#self.window1.get_widget("table4").set_sensitive()
		widgets = ['checkbutton12', 'label33', 'entry6', 'entry10']
		for w in widgets:
			self.window1.get_widget(w).set_sensitive(self.window1.get_widget("checkbutton7").get_active())



	def responseToDialog(self, entry, dialog, response):
		dialog.response(response)
	
	def getText(self, title, message, label, default = ''):
		#base this on a message dialog
		dialog = gtk.MessageDialog(
			None,
			gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
			gtk.MESSAGE_QUESTION,
			gtk.BUTTONS_OK_CANCEL,
			None
		)
		dialog.set_position(gtk.WIN_POS_CENTER_ALWAYS)
		dialog.set_markup(message)
		#create the text input field
		entry = gtk.Entry()
		entry.set_text(default)
		#allow the user to press enter to do ok
		entry.connect("activate", self.responseToDialog, dialog, gtk.RESPONSE_OK)
		#create a horizontal box to pack the entry and a label
		hbox = gtk.HBox()
		hbox.pack_start(gtk.Label(label), False, 5, 5)
		hbox.pack_end(entry)

		#add it and show it
		dialog.vbox.pack_end(hbox, True, True, 0)
		dialog.show_all()
		#go go go
		response = dialog.run()
		text = entry.get_text()
		dialog.destroy()
		
		if response == gtk.RESPONSE_OK:
			return text
		else:
			return None
	
	def save_preset(self, filename):
		self.get_settings()
		
		try:
			os.makedirs(os.path.dirname(filename))
		except:
			pass
			
		preset = ConfigParser.ConfigParser()
		preset.add_section('vidrop')
		for name in [
			'additional_mencoder_params'
			, 'auto_add_to_batch'
			, 'output_audio_bitrate'
			, 'output_audio_channels'
			, 'output_audio_codec'
			, 'output_audio_codec_settings'
			, 'output_audio_extrastereo'
			, 'output_audio_normalize_value'
			, 'output_audio_normalize'
			, 'output_audio_srate'
			, 'output_container_format'
			, 'output_fourcc'
			, 'output_video_bitrate'
			, 'output_video_blur_value'
			, 'output_video_blur'
			, 'output_video_codec'
			, 'output_video_codec_settings'
			, 'output_video_deinterlace'
			, 'output_video_flip'
			, 'output_video_fps'
			, 'output_video_height'
			, 'output_video_pp_brightness'
			, 'output_video_pp_contrast'
			, 'output_video_pp_gamma'
			, 'output_video_pp_saturation'
			, 'output_video_pp'
			, 'output_video_precision'
			, 'output_video_rotate'
			, 'output_video_width'
			, 'rebuild_index'
			, 'resize_method'
			, 'subtitle_antialias'
			, 'subtitle_ass'
			, 'subtitle_color'
			, 'subtitle_encoding'
			, 'subtitle_font'
			, 'subtitle_outline_color'
			, 'subtitle_position'
			, 'subtitle_size'
			, 'subtitle_width'
			, 'two_pass'
   			#, 'dvd_location'
			#, 'AUDIO_STREAMS_stream'
			#, 'dvd_subtitle_stream'
			#, 'dvd_title'
			#, 'input_dirname'
			#, 'input_file'
			#, 'input_subtitle'
			#, 'output_name'

			]:
			preset.set('vidrop', name, self.settings[name])

		try:
			configfile = open(filename, 'wb')
			#with open(filename, 'wb') as configfile:
			preset.write(configfile)
			configfile.close()
			return True
		except:
			return False
		return
			
	def combobox_select(self, cb, text, col = 0):
		model = cb.get_model()
		iter = model.get_iter_first()
		while iter:
			if model.get_value(iter,col) == text:
				cb.set_active_iter(iter)
				return		
			iter = model.iter_next(iter)
			
	def combobox_maketext(self,name):
		cb = self.window1.get_widget(name)
		liststore = gtk.ListStore(str)
		cb.set_model(liststore)
		cell = gtk.CellRendererText()
		cb.pack_start(cell, True)
		cb.add_attribute(cell, 'text', 0)  

	def load_preset(self, filename):
		#if not os.path.isfile(filename):
		#	return
		preset = CustomConfig()
		try:
			preset.read(filename)
		except:
			self.msg_info(_('Unable to load the selected preset. Loading default settings.'))

		self.window1.get_widget("checkbutton1").set_active(preset.get_boolean('vidrop', 'auto_add_to_batch', False))
		self.window1.get_widget("checkbutton2").set_active(preset.get_boolean('vidrop', 'rebuild_index', False))
		self.window1.get_widget("entry4").set_text(preset.getvalue('vidrop', 'output_video_fps', ''))

		output_container_format = preset.getvalue('vidrop', 'output_container_format', 'avi_noodml')
		for s in container_formats:
			if container_formats[s]['format'] == output_container_format:
				self.combobox_select(self.window1.get_widget("combobox1"),str(s))
		
		self.combobox_select(self.window1.get_widget("combobox2"),preset.getvalue('vidrop', 'output_video_codec', 'xvid'))
		
		self.window1.get_widget("entry11").set_text(preset.getvalue('vidrop', 'output_video_codec_settings', ':max_bframes=0:quant_type=h263:turbo:nochroma_me:notrellis:vhq=0'))
		
		
		self.window1.get_widget("comboboxentry13").child.set_text(preset.getvalue('vidrop', 'output_fourcc', 'XVID'))
		self.window1.get_widget("spinbutton1").set_value(preset.get_float('vidrop', 'output_video_width', 320.0))
		self.window1.get_widget("spinbutton2").set_value(preset.get_float('vidrop', 'output_video_height', 240.0))
		self.window1.get_widget("entry4").set_text(preset.getvalue('vidrop', 'output_video_fps', '22'))
		self.window1.get_widget("spinbutton4").set_value(preset.get_float('vidrop', 'output_video_bitrate', 300.0))
		self.window1.get_widget("spinbutton5").set_value(preset.get_float('vidrop', 'output_video_precision', 5.0))
		
		v = preset.getvalue('vidrop', 'resize_method', 'touch_outside')
		for s in resize_methods:
			if s == v:
				self.combobox_select(self.window1.get_widget("combobox6"),resize_methods[s])	
		
		self.combobox_select(self.window1.get_widget("combobox3"),preset.getvalue('vidrop', 'output_audio_codec', 'mp2'))
		self.window1.get_widget("entry12").set_text(preset.getvalue('vidrop', 'output_audio_codec_settings', ''))
		self.combobox_select(self.window1.get_widget("combobox4"),preset.getvalue('vidrop', 'output_audio_channels', '2'))
		self.combobox_select(self.window1.get_widget("combobox5"),preset.getvalue('vidrop', 'output_audio_bitrate', 128))
		self.combobox_select(self.window1.get_widget("combobox7"),preset.getvalue('vidrop', 'output_audio_srate', '44100'))
		self.window1.get_widget("checkbutton3").set_active(preset.get_boolean('vidrop', 'output_audio_normalize', False))
		self.window1.get_widget("spinbutton6").set_value(preset.get_float('vidrop', 'output_audio_normalize_value', 0.25))
		self.window1.get_widget("checkbutton11").set_active(preset.get_boolean('vidrop', 'output_audio_extrastereo', False))
		
		d = pango.FontDescription(preset.getvalue('vidrop', 'subtitle_font', 'Arial' if platform.system() == "Windows" else 'Sans' ))
		d.set_size(10 * pango.SCALE)
		self.window1.get_widget("fontbutton1").set_font_name(d.to_string())
		
		self.combobox_select(self.window1.get_widget("comboboxentry8"),preset.getvalue('vidrop', 'subtitle_encoding', 'cp1251'))
		self.window1.get_widget("checkbutton12").set_active(preset.get_boolean('vidrop', 'subtitle_antialias', False))
		self.window1.get_widget("checkbutton7").set_active(preset.get_boolean('vidrop', 'subtitle_ass', False))
		self.window1.get_widget("spinbutton7").set_value(preset.get_float('vidrop', 'subtitle_size', 5.0))
		self.window1.get_widget("spinbutton8").set_value(preset.get_float('vidrop', 'subtitle_position', 100.0))
		self.window1.get_widget("spinbutton9").set_value(preset.get_float('vidrop', 'subtitle_width', 100.0))
		self.window1.get_widget("entry6").set_text(preset.getvalue('vidrop', 'subtitle_color', 'FFFFFF00'))
		self.window1.get_widget("entry10").set_text(preset.getvalue('vidrop', 'subtitle_outline_color', '00000000'))
		self.window1.get_widget("entry9").set_text(preset.getvalue('vidrop', 'additional_mencoder_params', ''))
		self.window1.get_widget("checkbutton4").set_active(preset.get_boolean('vidrop', 'output_video_pp', False))
		self.window1.get_widget("spinbutton16").set_value(preset.get_float('vidrop', 'output_video_pp_gamma', 1.0))
		self.window1.get_widget("spinbutton17").set_value(preset.get_float('vidrop', 'output_video_pp_brightness', 0.05))
		self.window1.get_widget("spinbutton18").set_value(preset.get_float('vidrop', 'output_video_pp_contrast', 1.0))
		self.window1.get_widget("spinbutton19").set_value(preset.get_float('vidrop', 'output_video_pp_saturation', 1.0))
		
		v = preset.getvalue('vidrop', 'output_video_rotate', 'disabled')
		for s in rotate_methods:
			if s == v:
				self.combobox_select(self.window1.get_widget("combobox8"),rotate_methods[s])	
		self.window1.get_widget("checkbutton6").set_active(preset.get_boolean('vidrop', 'output_video_blur', False))
		self.window1.get_widget("spinbutton3").set_value(preset.get_float('vidrop', 'output_video_blur_value', 1.0))
		self.window1.get_widget("checkbutton8").set_active(preset.get_boolean('vidrop', 'output_video_flip', False))
		self.window1.get_widget("checkbutton9").set_active(preset.get_boolean('vidrop', 'two_pass', False))
		v = preset.getvalue('vidrop', 'output_video_deinterlace', 'disabled')
		for s in deinterlacers:
			if s == v:
				self.combobox_select(self.window1.get_widget("combobox9"),deinterlacers[s])	
				
		return
	
	def save_preferences(self):
		filename = os.path.join(self.abspath,'vidrop.ini')
		if platform.system() == "Windows":
			result = self.save_preset(filename)
		else:
			user_ini = os.path.join(os.getenv("HOME"),".local","share","vidrop","vidrop.ini")
			if not os.path.isfile(user_ini):
				if self.save_preset(filename):
					result = True
				else:
					result = self.save_preset(user_ini)
					filename = user_ini	
			else:
				result = self.save_preset(user_ini)
				filename = user_ini			
			
		if not result:
			return False
		try:
			preferences = ConfigParser.ConfigParser()
			preferences.read(filename)
			configfile = open(filename, 'wb')
			#with open(filename, 'wb') as configfile:
			preferences.set('vidrop', 'dvd_location', self.window1.get_widget("entry1").get_text())
			preferences.set('vidrop', 'show_log', self.window1.get_widget("checkbutton13").get_active())
			preferences.write(configfile)
			preferences.close()	
		except:
			return False
		
		return True
		
	def load_preferences(self):
		filename = os.path.join(self.abspath,'vidrop.ini')
		print "Loading ", filename
		if platform.system() == "Windows":
			filename = os.path.join(self.abspath,'vidrop.ini')
		else:
			filename = os.path.join(os.getenv("HOME"),".local","share","vidrop","vidrop.ini")
			if not os.path.isfile(filename):
				filename = os.path.join(self.abspath,'vidrop.ini')		
		
		#if not os.path.isfile(filename):
		#	return False
		self.load_preset(filename)
		preferences = CustomConfig()
		preferences.read(filename)
		self.window1.get_widget("entry1").set_text(preferences.getvalue('vidrop', 'dvd_location','E:' if platform.system() == "Windows" else '/dev/dvd'))
		self.window1.get_widget("checkbutton13").set_active(preferences.get_boolean('vidrop', 'show_log', False))
		#self.window1.get_widget("button20").modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse("#962A1C"))
		return True
	
	# TODO
	def calc_output_size(self):
		return 0
		
	def launch_browser(self, url):
		if platform.system() == "Windows":
			pass #os.system()
		else:
			os.system('xdg-open "%s"' % url)			
class Namespace: pass

class CustomConfig( ConfigParser.ConfigParser ):
	def getvalue(self,section,name,default = ''):
		try:
			value = self.get(section,name)
		except:
			value = default
		return value
		
	def get_boolean(self,section,name,default = ''):
		try:
			value = self.getboolean(section,name)
		except:
			value = default
		return value	

	def get_float(self,section,name,default = ''):
		try:
			value = self.getfloat(section,name)
		except:
			value = default
		return value	

		
if __name__ == '__main__':
	vidrop()
	gtk.main()
	
	

